@@ -27,12 +27,24 @@ from bzrlib import (
     )
 
 from bzrlib.plugins.svn.cache import (
+    CacheConcurrencyError,
     RepositoryCache,
-    cachedbs,
     )
 from bzrlib.plugins.svn.mapping import (
     mapping_registry,
     )
+from bzrlib.plugins.svn.revids import (
+    RevisionIdMapCache,
+    )
+from bzrlib.plugins.svn.revmeta import (
+    RevisionInfoCache,
+    )
+from bzrlib.plugins.svn.logwalker import (
+    LogCache,
+    )
+from bzrlib.plugins.svn.parents import (
+    ParentsCache,
+    )
 
 from subvertpy import NODE_UNKNOWN
 
@@ -59,7 +71,11 @@ except:
     raise errors.BzrError("missing sqlite library")
 
 
-connect_cachefile = sqlite3.connect
+def _connect_sqlite3_file(path):
+    return sqlite3.connect(path, timeout=20.0, isolation_level=None)
+
+
+connect_cachefile = _connect_sqlite3_file
 
 
 class CacheTable(object):
@@ -72,15 +88,39 @@ class CacheTable(object):
             self.cachedb = cache_db
         self._commit_interval = 500
         self._create_table()
-        self.cachedb.commit()
-        self._commit_countdown = self._commit_interval
+        self.commit()
+
+    @classmethod
+    def _convert_operational_errors(cls, e):
+        if "database is locked" in e:
+            return CacheConcurrencyError()
+        else:
+            return e
+
+    def execute(self, *args, **kwargs):
+        try:
+            return self.cachedb.execute(*args, **kwargs)
+        except sqlite3.OperationalError, e:
+            raise self._convert_operational_errors(e)
+
+    def executemany(self, *args, **kwargs):
+        try:
+            return self.cachedb.executemany(*args, **kwargs)
+        except sqlite3.OperationalError, e:
+            raise self._convert_operational_errors(e)
+
+    def executescript(self, *args, **kwargs):
+        try:
+            return self.cachedb.executescript(*args, **kwargs)
+        except sqlite3.OperationalError, e:
+            raise self._convert_operational_errors(e)
 
     def commit(self):
         """Commit the changes to the database."""
         self.cachedb.commit()
         self._commit_countdown = self._commit_interval
 
-    def commit_conditionally(self):
+    def _commit_conditionally(self):
         self._commit_countdown -= 1
         if self._commit_countdown <= 0:
             self.commit()
@@ -97,11 +137,7 @@ class CacheTable(object):
 CACHE_DB_VERSION = 5
 
 
-class RevisionIdMapCache(CacheTable):
-    """Revision id mapping store.
-
-    Stores mapping from revid -> (path, revnum, mapping)
-    """
+class SqliteRevisionIdMapCache(RevisionIdMapCache, CacheTable):
 
     def _create_table(self):
         self.cachedb.executescript("""
@@ -126,39 +162,30 @@ class RevisionIdMapCache(CacheTable):
         self._commit_interval = 500
 
     def set_last_revnum_checked(self, layout, revnum):
-        """Remember the latest revision number that has been checked
-        for a particular layout.
+        """See RevisionIdMapCache.set_last_revnum_checked."""
 
-        :param layout: Repository layout.
-        :param revnum: Revision number.
-        """
-        self.cachedb.execute("replace into revids_seen (layout, max_revnum) VALUES (?, ?)", (layout, revnum))
-        self.commit_conditionally()
+        self.mutter("set last revnum checked for %r to %r", layout, revnum)
+        self.execute("replace into revids_seen (layout, max_revnum) VALUES (?, ?)", (layout, revnum))
+        self.commit()
 
     def last_revnum_checked(self, layout):
-        """Retrieve the latest revision number that has been checked
-        for revision ids for a particular layout.
+        """See RevisionIdMapCache.last_revnum_checked."""
 
-        :param layout: Repository layout.
-        :return: Last revision number checked or 0.
-        """
-        self.mutter("last revnum checked %r", layout)
-        ret = self.cachedb.execute(
+        ret = self.execute(
             "select max_revnum from revids_seen where layout = ?", (layout,)).fetchone()
         if ret is None:
-            return 0
-        return int(ret[0])
+            revnum = 0
+        else:
+            revnum = int(ret[0])
+        self.mutter("last revnum checked for %r is %r", layout, revnum)
+        return revnum
 
     def lookup_revid(self, revid):
-        """Lookup the details for a particular revision id.
+        """See RevisionIdMapCache.lookup_revid."""
 
-        :param revid: Revision id.
-        :return: Tuple with path inside repository, minimum revision number, maximum revision number and
-            mapping.
-        """
         assert isinstance(revid, str)
         self.mutter("lookup revid %r", revid)
-        ret = self.cachedb.execute(
+        ret = self.execute(
             "select path, min_revnum, max_revnum, mapping from revmap where revid=? order by abs(min_revnum-max_revnum) asc", (revid,)).fetchone()
         if ret is None:
             raise errors.NoSuchRevision(self, revid)
@@ -169,16 +196,12 @@ class RevisionIdMapCache(CacheTable):
             return (path, min_revnum, max_revnum, mapping)
 
     def lookup_branch_revnum(self, revnum, path, mapping):
-        """Lookup a revision by revision number, branch path and mapping.
+        """See RevisionIdMapCache.lookup_branch_revnum."""
 
-        :param revnum: Subversion revision number.
-        :param path: Subversion branch path.
-        :param mapping: Mapping
-        """
         assert isinstance(revnum, int)
         assert isinstance(path, str)
         assert isinstance(mapping, str)
-        row = self.cachedb.execute(
+        row = self.execute(
                 "select revid from revmap where max_revnum=? and min_revnum=? and path=? and mapping=?", (revnum, revnum, path, mapping)).fetchone()
         if row is not None:
             ret = str(row[0])
@@ -188,17 +211,8 @@ class RevisionIdMapCache(CacheTable):
         return ret
 
     def insert_revid(self, revid, branch, min_revnum, max_revnum, mapping):
-        """Insert a revision id into the revision id cache.
+        """See RevisionIdMapCache.insert_revid."""
 
-        :param revid: Revision id for which to insert metadata.
-        :param branch: Branch path at which the revision was seen
-        :param min_revnum: Minimum Subversion revision number in which the
-                           revid was found
-        :param max_revnum: Maximum Subversion revision number in which the
-                           revid was found
-        :param mapping: Name of the mapping with which the revision
-                       was found
-        """
         assert revid is not None and revid != ""
         assert isinstance(mapping, str)
         assert isinstance(branch, str)
@@ -206,23 +220,24 @@ class RevisionIdMapCache(CacheTable):
         assert min_revnum <= max_revnum
         self.mutter("insert revid %r:%r-%r -> %r", branch, min_revnum, max_revnum, revid)
         if min_revnum == max_revnum:
-            cursor = self.cachedb.execute(
+            cursor = self.execute(
                 "update revmap set min_revnum = ?, max_revnum = ? WHERE revid=? AND path=? AND mapping=?",
                 (min_revnum, max_revnum, revid, branch, mapping))
         else:
-            cursor = self.cachedb.execute(
+            cursor = self.execute(
                 "update revmap set min_revnum = MAX(min_revnum,?), max_revnum = MIN(max_revnum, ?) WHERE revid=? AND path=? AND mapping=?",
                 (min_revnum, max_revnum, revid, branch, mapping))
         if cursor.rowcount == 0:
-            self.cachedb.execute(
-                "insert into revmap (revid,path,min_revnum,max_revnum,mapping) VALUES (?,?,?,?,?)",
+            self.execute(
+                "replace into revmap (revid,path,min_revnum,max_revnum,mapping) VALUES (?,?,?,?,?)",
                 (revid, branch, min_revnum, max_revnum, mapping))
+        self._commit_conditionally()
 
 
-class RevisionInfoCache(CacheTable):
+class SqliteRevisionInfoCache(RevisionInfoCache, CacheTable):
 
     def _create_table(self):
-        self.cachedb.executescript("""
+        self.executescript("""
         create table if not exists revmetainfo (
             path text not null,
             revnum integer,
@@ -244,33 +259,26 @@ class RevisionInfoCache(CacheTable):
         self._commit_interval = 500
 
     def set_original_mapping(self, foreign_revid, original_mapping):
+        """See RevisionInfoCache.set_original_mapping."""
+
         if original_mapping is not None:
             orig_mapping_name = original_mapping.name
         else:
             orig_mapping_name = None
-        self.cachedb.execute("insert into original_mapping (path, revnum, original_mapping) values (?, ?, ?)", (foreign_revid[1], foreign_revid[2], orig_mapping_name))
+        self.execute("replace into original_mapping (path, revnum, original_mapping) values (?, ?, ?)", (foreign_revid[1].decode("utf-8"), foreign_revid[2], orig_mapping_name))
 
     def insert_revision(self, foreign_revid, mapping, (revno, revid, hidden),
             stored_lhs_parent_revid):
-        """Insert a revision to the cache.
+        """See RevisionInfoCache.insert_revision."""
 
-        :param foreign_revid: Foreign revision id
-        :param mapping: Mapping used
-        :param info: Tuple with (revno, revid, hidden)
-        :param stored_lhs_parent_revid: Stored lhs parent revision
-        """
-        self.cachedb.execute("insert into revmetainfo (path, revnum, mapping, revid, revno, hidden, stored_lhs_parent_revid) values (?, ?, ?, ?, ?, ?, ?)", (foreign_revid[1], foreign_revid[2], mapping.name, revid, revno, hidden, stored_lhs_parent_revid))
+        self.execute("replace into revmetainfo (path, revnum, mapping, revid, revno, hidden, stored_lhs_parent_revid) values (?, ?, ?, ?, ?, ?, ?)", (foreign_revid[1], foreign_revid[2], mapping.name, revid, revno, hidden, stored_lhs_parent_revid))
+        self._commit_conditionally()
 
     def get_revision(self, foreign_revid, mapping):
-        """Get the revision metadata info for a (foreign_revid, mapping) tuple.
+        """See RevisionInfoCache.get_revision."""
 
-        :param foreign_revid: Foreign revision id
-        :param mapping: Mapping
-        :return: Tuple with (stored revno, revid, hidden),
-            stored_lhs_parent_revid
-        """
         # Will raise KeyError if not present
-        row = self.cachedb.execute("select revno, revid, hidden, stored_lhs_parent_revid from revmetainfo where path = ? and revnum = ? and mapping = ?", (foreign_revid[1], foreign_revid[2], mapping.name)).fetchone()
+        row = self.execute("select revno, revid, hidden, stored_lhs_parent_revid from revmetainfo where path = ? and revnum = ? and mapping = ?", (foreign_revid[1], foreign_revid[2], mapping.name)).fetchone()
         if row is None:
             raise KeyError((foreign_revid, mapping))
         else:
@@ -285,12 +293,9 @@ class RevisionInfoCache(CacheTable):
             return ((row[0], stored_revid, row[2]), stored_lhs_parent_revid)
 
     def get_original_mapping(self, foreign_revid):
-        """Find the original mapping for a revision.
+        """See RevisionInfoCache.get_original_mapping."""
 
-        :param foreign_revid: Foreign revision id
-        :return: Mapping object or None
-        """
-        row = self.cachedb.execute("select original_mapping from original_mapping where path = ? and revnum = ?", (foreign_revid[1], foreign_revid[2])).fetchone()
+        row = self.execute("select original_mapping from original_mapping where path = ? and revnum = ?", (foreign_revid[1].decode("utf-8"), foreign_revid[2])).fetchone()
         if row is None:
             raise KeyError(foreign_revid)
         if row[0] is None:
@@ -299,13 +304,10 @@ class RevisionInfoCache(CacheTable):
             return mapping_registry.parse_mapping_name("svn-" + row[0].encode("utf-8"))
 
 
-class LogCache(CacheTable):
-    """Log browser cache table manager. The methods of this class
-    encapsulate the SQL commands used by CachingLogWalker to access
-    the log cache tables."""
+class SqliteLogCache(LogCache, CacheTable):
 
     def _create_table(self):
-        self.cachedb.executescript("""
+        self.executescript("""
             create table if not exists changed_path(
                 rev integer,
                 action text not null check(action in ('A', 'D', 'M', 'R')),
@@ -328,15 +330,17 @@ class LogCache(CacheTable):
             create unique index if not exists revinfo_rev on revinfo(rev);
         """ % NODE_UNKNOWN)
         try:
-            self.cachedb.executescript(
+            self.executescript(
                 "alter table changed_path ADD kind INT DEFAULT %d;" % NODE_UNKNOWN)
         except sqlite3.OperationalError:
             pass # Column already exists.
 
     def find_latest_change(self, path, revnum):
+        """See LogCache.find_latest_change."""
+
         if path == "":
-            return self.cachedb.execute("select max(rev) from changed_path where rev <= ?", (revnum,)).fetchone()[0]
-        return self.cachedb.execute("""
+            return self.execute("select max(rev) from changed_path where rev <= ?", (revnum,)).fetchone()[0]
+        return self.execute("""
             select max(rev) from changed_path
             where rev <= ?
             and (path=?
@@ -346,11 +350,9 @@ class LogCache(CacheTable):
         """, (revnum, path, path + "/*", path)).fetchone()[0]
 
     def get_revision_paths(self, revnum):
-        """Return all history information for a given revision number.
+        """See LogCache.get_revision_paths."""
 
-        :param revnum: Revision number of revision.
-        """
-        result = self.cachedb.execute("select path, action, copyfrom_path, copyfrom_rev, kind from changed_path where rev=?", (revnum,))
+        result = self.execute("select path, action, copyfrom_path, copyfrom_rev, kind from changed_path where rev=?", (revnum,))
         paths = {}
         for p, act, cf, cr, kind in result:
             if cf is not None:
@@ -358,12 +360,10 @@ class LogCache(CacheTable):
             paths[p.encode("utf-8")] = (act, cf, cr, kind)
         return paths
 
-    def insert_paths(self, rev, orig_paths):
-        """Insert new history information into the cache.
+    def insert_paths(self, rev, orig_paths, revprops, all_revprops):
+        """See LogCache.insert_paths."""
+        self.insert_revprops(rev, revprops, all_revprops)
 
-        :param rev: Revision number of the revision
-        :param orig_paths: SVN-style changes dictionary
-        """
         if orig_paths is None or orig_paths == {}:
             return
         new_paths = []
@@ -378,19 +378,19 @@ class LogCache(CacheTable):
                 kind = NODE_UNKNOWN
             new_paths.append((rev, p.strip("/").decode("utf-8"), v[0], copyfrom_path, v[2], kind))
 
-        self.cachedb.executemany("replace into changed_path (rev, path, action, copyfrom_path, copyfrom_rev, kind) values (?, ?, ?, ?, ?, ?)", new_paths)
+        self.executemany("replace into changed_path (rev, path, action, copyfrom_path, copyfrom_rev, kind) values (?, ?, ?, ?, ?, ?)", new_paths)
 
     def drop_revprops(self, revnum):
-        self.cachedb.execute("update revinfo set all_revprops = 0 where rev = ?", (revnum,))
+        """See LogCache.drop_revprops."""
+
+        self.execute("update revinfo set all_revprops = 0 where rev = ?", (revnum,))
 
     def get_revprops(self, revnum):
-        """Retrieve all the cached revision properties.
+        """See LogCache.get_revprops."""
 
-        :param revnum: Revision number of revision to retrieve revprops for.
-        """
-        result = self.cachedb.execute("select name, value from revprop where rev = ?", (revnum,))
+        result = self.execute("select name, value from revprop where rev = ?", (revnum,))
         revprops = dict((k.encode("utf-8"), v.encode("utf-8")) for (k, v) in result)
-        result = self.cachedb.execute("select all_revprops from revinfo where rev = ?", (revnum,)).fetchone()
+        result = self.execute("select all_revprops from revinfo where rev = ?", (revnum,)).fetchone()
         if result is None:
             all_revprops = False
         else:
@@ -398,33 +398,33 @@ class LogCache(CacheTable):
         return (revprops, all_revprops)
 
     def insert_revprops(self, revision, revprops, all_revprops):
+        """See LogCache.insert_revprops."""
+
         if revprops is None:
             return
-        self.cachedb.executemany("replace into revprop (rev, name, value) values (?, ?, ?)", [(revision, name.decode("utf-8", "replace"), value.decode("utf-8", "replace")) for (name, value) in revprops.iteritems()])
-        self.cachedb.execute("""
+        self.executemany("replace into revprop (rev, name, value) values (?, ?, ?)", [(revision, name.decode("utf-8", "replace"), value.decode("utf-8", "replace")) for (name, value) in revprops.iteritems()])
+        self.execute("""
             replace into revinfo (rev, all_revprops) values (?, ?)
         """, (revision, all_revprops))
 
-    def has_all_revprops(self, revnum):
-        """Check whether all revprops for a revision have been cached.
+    def max_revnum(self):
+        """See LogCache.last_revnum."""
 
-        :param revnum: Revision number of the revision.
-        """
-
-    def last_revnum(self):
-        saved_revnum = self.cachedb.execute("SELECT MAX(rev) FROM changed_path").fetchone()[0]
+        saved_revnum = self.execute("SELECT MAX(rev) FROM changed_path").fetchone()[0]
         if saved_revnum is None:
             return 0
         return saved_revnum
 
     def min_revnum(self):
-        return self.cachedb.execute("SELECT MIN(rev) FROM revprop").fetchone()[0]
+        """See LogCache.min_revnum."""
+
+        return self.execute("SELECT MIN(rev) FROM changed_path").fetchone()[0]
 
 
-class ParentsCache(CacheTable):
+class SqliteParentsCache(ParentsCache, CacheTable):
 
     def _create_table(self):
-        self.cachedb.executescript("""
+        self.executescript("""
         create table if not exists parent (
             rev text not null,
             parent text,
@@ -436,16 +436,21 @@ class ParentsCache(CacheTable):
         self._commit_interval = 200
 
     def insert_parents(self, revid, parents):
+        """See ParentsCache.insert_parents."""
+
         self.mutter('insert parents: %r -> %r', revid, parents)
         if len(parents) == 0:
-            self.cachedb.execute("replace into parent (rev, parent, idx) values (?, NULL, -1)", (revid,))
+            self.execute("replace into parent (rev, parent, idx) values (?, NULL, -1)", (revid,))
         else:
             for i, p in enumerate(parents):
-                self.cachedb.execute("replace into parent (rev, parent, idx) values (?, ?, ?)", (revid, p, i))
+                self.execute("replace into parent (rev, parent, idx) values (?, ?, ?)", (revid, p, i))
+        self._commit_conditionally()
 
     def lookup_parents(self, revid):
+        """See ParentsCache.lookup_parents."""
+
         self.mutter('lookup parents: %r', revid)
-        rows = self.cachedb.execute("select parent from parent where rev = ? order by idx", (revid, )).fetchall()
+        rows = self.execute("select parent from parent where rev = ? order by idx", (revid, )).fetchall()
         if len(rows) == 0:
             return None
         return tuple([row[0].encode("utf-8") for row in rows if row[0] is not None])
@@ -454,21 +459,23 @@ class ParentsCache(CacheTable):
 class SqliteRepositoryCache(RepositoryCache):
     """Object that provides a cache related to a particular UUID."""
 
-    def open_sqlite(self):
+    def __init__(self, uuid):
+        super(SqliteRepositoryCache, self).__init__(uuid)
         cache_file = os.path.join(self.create_cache_dir(), 'cache-v%d' % CACHE_DB_VERSION)
         assert isinstance(cache_file, str)
-        if not cachedbs().has_key(cache_file):
-            cachedbs()[cache_file] = connect_cachefile(cache_file.decode(osutils._fs_enc).encode("utf-8"))
-        return cachedbs()[cache_file]
+        self._sqlite = connect_cachefile(cache_file.decode(osutils._fs_enc).encode("utf-8"))
 
     def open_revid_map(self):
-        return RevisionIdMapCache(self.open_sqlite())
+        return SqliteRevisionIdMapCache(self._sqlite)
 
     def open_logwalker(self):
-        return LogCache(self.open_sqlite())
+        return SqliteLogCache(self._sqlite)
 
     def open_revision_cache(self):
-        return RevisionInfoCache(self.open_sqlite())
+        return SqliteRevisionInfoCache(self._sqlite)
 
     def open_parents(self):
-        return ParentsCache(self.open_sqlite())
+        return SqliteParentsCache(self._sqlite)
+
+    def commit(self):
+        self._sqlite.commit()
