@@ -23,15 +23,16 @@ from subvertpy import (
     properties,
     )
 
-import bzrlib.config
 from bzrlib import (
     atomicfile,
+    config as _mod_bzr_config,
     osutils,
     trace,
+    transport,
+    version_info as bzrlib_version,
     )
 from bzrlib.config import (
     ConfigObj,
-    IniBasedConfig,
     config_dir,
     ensure_config_dir_exists,
     GlobalConfig,
@@ -46,6 +47,9 @@ from bzrlib.errors import (
     )
 
 
+get_transport_from_path = getattr(transport, "get_transport_from_path", transport.get_transport) # bzr < 2.5
+
+
 def as_bool(str):
     """Parse a string as a boolean.
 
@@ -69,24 +73,81 @@ def as_bool(str):
 # Data stored includes default branching scheme and locations the repository
 # was seen at.
 
-def subversion_config_filename():
-    """Return per-user configuration ini file filename."""
-    return osutils.pathjoin(config_dir(), 'subversion.conf')
+
+class SubversionStore(_mod_bzr_config.LockableIniFileStore):
+
+    def __init__(self, possible_transports=None):
+        t = get_transport_from_path(
+            _mod_bzr_config.config_dir(),
+            possible_transports=possible_transports)
+        super(SubversionStore, self).__init__(t, 'subversion.conf')
 
 
-config_base_class = getattr(bzrlib.config, "LockableConfig", IniBasedConfig)
+class UUIDMatcher(_mod_bzr_config.SectionMatcher):
+    """UUID-based Subversion section matcher."""
+
+    def __init__(self, store, uuid):
+        super(UUIDMatcher, self).__init__(store)
+        self.uuid = uuid
+
+    def match(self, section):
+        return section.id == self.uuid
 
 
-class SubversionUUIDConfig(config_base_class):
+class SvnBranchStack(_mod_bzr_config._CompatibleStack):
+    """SvnBranch stack providing UUID specific options."""
+
+    def __init__(self, branch):
+        lstore = _mod_bzr_config.LocationStore()
+        loc_matcher = _mod_bzr_config.LocationMatcher(lstore, branch.base)
+        svn_store = SubversionStore()
+        uuid_matcher = UUIDMatcher(svn_store,
+                                   getattr(branch.repository, 'uuid', None))
+        gstore = _mod_bzr_config.GlobalStore()
+        super(SvnBranchStack, self).__init__(
+            [self._get_overrides,
+             loc_matcher.get_sections,
+             uuid_matcher.get_sections,
+             gstore.get_sections],
+            # All modifications go to the corresponding section in
+            # locations.conf
+            lstore, branch.base)
+        self.branch = branch
+
+
+class SvnRepositoryStack(_mod_bzr_config._CompatibleStack):
+    """SvnRepository stack providing UUID specific options."""
+
+    def __init__(self, repository):
+        lstore = _mod_bzr_config.LocationStore()
+        loc_matcher = _mod_bzr_config.LocationMatcher(lstore, repository.base)
+        svn_store = SubversionStore()
+        uuid_matcher = UUIDMatcher(svn_store,
+                                   getattr(repository, 'uuid', None))
+        gstore = _mod_bzr_config.GlobalStore()
+        super(SvnRepositoryStack, self).__init__(
+            [self._get_overrides,
+             loc_matcher.get_sections,
+             uuid_matcher.get_sections,
+             gstore.get_sections],
+            # All modifications go to the corresponding section in
+            # locations.conf
+            lstore, repository.base)
+        self.repository = repository
+
+
+class SubversionUUIDConfig(_mod_bzr_config.LockableConfig):
     """UUID-based Subversion configuration."""
 
     def __init__(self, uuid):
-        name_generator = subversion_config_filename
-        super(SubversionUUIDConfig, self).__init__(name_generator)
+        super(SubversionUUIDConfig, self).__init__(self._get_filename())
         self.uuid = uuid
         if not self.uuid in self._get_parser():
             self._get_parser()[self.uuid] = {}
 
+    def _get_filename(self):
+        return osutils.pathjoin(config_dir(), 'subversion.conf')
+
     def __getitem__(self, name):
         return self._get_parser()[self.uuid][name]
 
@@ -248,14 +309,6 @@ class SvnRepositoryConfig(Config):
             return BranchingScheme.find_scheme(schemename.encode('ascii'))
         return None
 
-    def get_supports_change_revprop(self):
-        """Check whether or not the repository supports changing existing
-        revision properties."""
-        try:
-            return self.get_bool("supports-change-revprop")
-        except KeyError:
-            return None
-
     def get_use_cache(self):
         try:
             if self.get_bool("use-cache"):
@@ -301,6 +354,11 @@ class SvnRepositoryConfig(Config):
         locations.add(location.rstrip("/"))
         self.set_user_option('locations', ";".join(list(locations)))
 
+    def get_default_stack_on(self):
+        return None
+
+    def set_default_stack_on(self, value):
+        raise BzrError("Cannot set configuration")
 
 
 class BranchConfig(Config):
@@ -372,7 +430,16 @@ class BranchConfig(Config):
                 return value
         return None
 
-    def get_append_revisions_only(self, default=None):
+    def get_allow_metadata_in_fileprops(self):
+        """Check whether file properties may be used to store bzr-svn metadata.
+
+        """
+        try:
+            return self.get_bool("allow_metadata_in_file_properties")
+        except KeyError:
+            return False
+
+    def get_append_revisions_only(self, default=True):
         """Check whether it is possible to remove revisions from the mainline.
         """
         try:
@@ -417,6 +484,9 @@ class BranchConfig(Config):
         except KeyError:
             return None
 
+    def has_explicit_nickname(self):
+        return False
+
 
 class PropertyConfig(object):
     """ConfigObj-like class that looks at Subversion file ids."""
@@ -456,7 +526,9 @@ class SubversionBuildPackageConfig(object):
             self.wt_layout_path = os.path.join(tree.abspath("."), ".svn", "svn-layout")
             self.option_source = ConfigObj(self.wt_layout_path, encoding="utf-8")
         elif tree.has_filename("debian/svn-layout"):
-            self.option_source = ConfigObj(tree.get_file(tree.path2id("debian/svn-layout"), "debian/svn-layout"), encoding="utf-8")
+            file_id = tree.path2id("debian/svn-layout")
+            layout_file = tree.get_file(file_id, "debian/svn-layout")
+            self.option_source = ConfigObj(layout_file, encoding="utf-8")
         elif isinstance(tree, SubversionTree) and tree.has_filename("debian"):
             self.option_source = PropertyConfig(tree, "debian", "svn-bp:")
         else:
@@ -487,10 +559,29 @@ class SubversionBuildPackageConfig(object):
         self.option_source[option_name] = value
 
 
-class SubversionControlDirConfig(object):
+if bzrlib_version < (2, 5):
+    def Option(*args, **kwargs):
+        if 'help' in kwargs:
+            del kwargs['help']
+        return _mod_bzr_config.Option(*args, **kwargs)
+else:
+    Option = _mod_bzr_config.Option
 
-    def get_default_stack_on(self):
-        return None
 
-    def set_default_stack_on(self, value):
-        raise BzrError("Cannot set configuration")
+svn_layout_option = Option('svn.layout', default=None,
+           help='''\
+The mandated bzr-svn repository layout to use.
+
+See `bzr help svn-layout` for details.
+''')
+
+svn_branches_option = Option('branches', default=None,
+           help='''\
+Paths in the Subversion repository to consider branches.
+
+This should be a comma-separated list of paths, each relative to the
+repository root. Asterisks may be used as wildcards, and will match
+a single path element.
+
+See `bzr help svn-layout` for more information on Subversion repository layouts.
+''')
