@@ -13,7 +13,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from copy import copy
 import subvertpy
 from subvertpy import ra
 
@@ -24,10 +23,18 @@ from bzrlib.errors import (
     NoSuchRevision,
     )
 from bzrlib.repository import Repository
-from bzrlib.tests import TestCase
+from bzrlib.tests import (
+    TestCase,
+    TestNotApplicable,
+    )
 from bzrlib.workingtree import WorkingTree
 
-from bzrlib.plugins.svn import format
+from bzrlib.plugins.svn.config import (
+    BranchConfig,
+    )
+from bzrlib.plugins.svn.errors import (
+    RequiresMetadataInFileProps,
+    )
 from bzrlib.plugins.svn.layout.standard import (
     RootLayout,
     TrunkLayout,
@@ -126,16 +133,13 @@ class RepositoryTests(SubversionTestCase):
 
     def setUp(self):
         super(RepositoryTests, self).setUp()
-        self.repos_url = self.make_repository("d")
+        self.repos_url = self.make_svn_repository("d")
         self._old_mapping = mapping_registry._get_default_key()
-        mapping_registry.set_default("v3")
-
-    def tearDown(self):
-        super(RepositoryTests, self).tearDown()
+        self.addCleanup(mapping_registry.set_default, self._old_mapping)
         mapping_registry.set_default("v3")
 
     def test_revision_id_to_revno_simple(self):
-        repos_url = self.make_repository('a')
+        repos_url = self.make_svn_repository('a')
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
@@ -147,7 +151,7 @@ class RepositoryTests(SubversionTestCase):
         self.assertEquals(2, branch.revision_id_to_revno("myrevid"))
 
     def test_revision_id_to_revno_older(self):
-        repos_url = self.make_repository('a')
+        repos_url = self.make_svn_repository('a')
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
@@ -240,31 +244,6 @@ class RepositoryTests(SubversionTestCase):
         self.assertEqual(author, rev.committer)
         self.assertIsInstance(rev.properties, dict)
 
-    def test_get_ancestry_merged(self):
-        self.make_checkout(self.repos_url, 'dc')
-        self.build_tree({'dc/foo': "data"})
-        self.client_add("dc/foo")
-        self.client_commit("dc", "My Message")
-        self.client_update("dc")
-        self.client_set_prop("dc", "bzr:ancestry:v3-none", "a-parent\n")
-        self.build_tree({'dc/foo': "data2"})
-        self.client_commit("dc", "Second Message")
-        repository = Repository.open(self.repos_url)
-        mapping = repository.get_mapping()
-        self.assertEqual([None, repository.generate_revision_id(0, "", mapping)],
-                repository.get_ancestry(
-                    repository.generate_revision_id(0, "", mapping)))
-        self.assertEqual([None, repository.generate_revision_id(0, "", mapping),
-            repository.generate_revision_id(1, "", mapping)],
-                repository.get_ancestry(
-                    repository.generate_revision_id(1, "", mapping)))
-        self.assertEqual([None, 
-            repository.generate_revision_id(0, "", mapping), "a-parent", 
-            repository.generate_revision_id(1, "", mapping), 
-                  repository.generate_revision_id(2, "", mapping)], 
-                repository.get_ancestry(
-                    repository.generate_revision_id(2, "", mapping)))
-
     def test_lookup_revision_id_overridden(self):
         dc = self.get_commit_editor(self.repos_url)
         dc.add_dir("bloe")
@@ -354,7 +333,7 @@ class RepositoryTests(SubversionTestCase):
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
         mapping = oldrepos.get_mapping()
-        self.assertEqual("bla", newrepos.get_inventory(
+        self.assertEqual("bla", newrepos.revision_tree(
             oldrepos.generate_revision_id(1, "", mapping)).path2id("test"))
 
     def test_fetch_ghosts(self):
@@ -458,25 +437,23 @@ class RepositoryTests(SubversionTestCase):
         repository = Repository.open(self.repos_url)
         repository.set_layout(RootLayout())
         tree = repository.revision_tree(Branch.open(self.repos_url).last_revision())
-        self.assertEqual("someid", tree.inventory.path2id("foo"))
-        self.assertFalse("1@%s::foo" % repository.uuid in tree.inventory)
+        self.assertEqual("someid", tree.path2id("foo"))
+        self.assertFalse(tree.has_id("1@%s::foo" % repository.uuid))
 
     def test_commit_revision_id(self):
         self.make_checkout(self.repos_url, "dc")
         wt = WorkingTree.open("dc")
+        wt.branch.get_config().set_user_option("allow_metadata_in_file_properties", "True")
         self.build_tree({'dc/foo/bla': "data", 'dc/bla': "otherdata"})
         wt.add('bla')
         wt.commit(message="data")
-
         branch = Branch.open(self.repos_url)
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
         builder = branch.get_commit_builder([branch.last_revision()], 
                 revision_id="my-revision-id")
         tree = branch.repository.revision_tree(branch.last_revision())
-        new_tree = copy(tree)
-        ie = new_tree.inventory.root
-        ie.revision = None
-        builder.record_entry_contents(ie, [tree.inventory], '', new_tree, 
-                                      None)
+        list(builder.record_iter_changes(tree, branch.last_revision(), []))
         builder.finish_inventory()
         builder.commit("foo")
 
@@ -484,23 +461,34 @@ class RepositoryTests(SubversionTestCase):
             self.client_get_prop("dc", 
                 "bzr:revision-id:v3-none", 2))
 
+    def test_commit_metadata_requires_config_change(self):
+        self.make_checkout(self.repos_url, "dc")
+
+        wt = WorkingTree.open("dc")
+        self.build_tree({'dc/foo/bla': "data", 'dc/bla': "otherdata"})
+        wt.add('bla')
+        self.assertRaises(RequiresMetadataInFileProps, wt.commit,
+            message="data")
+        wt.branch.get_config().set_user_option("allow_metadata_in_file_properties", "True")
+        wt.commit(message="data")
+
     def test_commit_metadata(self):
         self.make_checkout(self.repos_url, "dc")
 
         wt = WorkingTree.open("dc")
         self.build_tree({'dc/foo/bla': "data", 'dc/bla': "otherdata"})
         wt.add('bla')
+        wt.branch.get_config().set_user_option("allow_metadata_in_file_properties", "True")
         wt.commit(message="data")
 
         branch = Branch.open(self.repos_url)
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
         builder = branch.get_commit_builder([branch.last_revision()], 
                 timestamp=4534.0, timezone=2, committer="fry",
                 revision_id="my-revision-id")
         tree = branch.repository.revision_tree(branch.last_revision())
-        new_tree = copy(tree)
-        ie = new_tree.inventory.root
-        ie.revision = None
-        builder.record_entry_contents(ie, [tree.inventory], '', new_tree, None)
+        list(builder.record_iter_changes(tree, branch.last_revision(), []))
         builder.finish_inventory()
         builder.commit("foo")
 
@@ -516,6 +504,8 @@ class RepositoryTests(SubversionTestCase):
         self.build_tree({'dc/foo/bla': "data"})
         self.client_add("dc/foo")
         wt = WorkingTree.open("dc")
+        wt.branch.get_config().set_user_option("allow_metadata_in_file_properties", "True")
+
         wt.set_pending_merges(["some-ghost-revision"])
         self.assertEqual(["some-ghost-revision"], wt.get_parent_ids()[1:])
         wt.commit(message="data")
@@ -525,11 +515,11 @@ class RepositoryTests(SubversionTestCase):
                          wt.branch.repository.get_revision(
                              wt.branch.last_revision()).parent_ids)
 
-    def test_push_unnecessary_merge(self):        
+    def test_push_unnecessary_merge(self):
         from bzrlib.debug import debug_flags
         debug_flags.add("commit")
         debug_flags.add("fetch")
-        repos_url = self.make_repository("a")
+        repos_url = self.make_svn_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c")
         self.build_tree({'c/registry/generic.c': "Tour"})
         bzrwt.add("registry")
@@ -539,6 +529,9 @@ class RepositoryTests(SubversionTestCase):
 
         # Push first branch into Subversion
         newdir = BzrDir.open(repos_url+"/trunk")
+        config = BranchConfig(repos_url+"/trunk", newdir.find_repository().uuid)
+        config.set_user_option("allow_metadata_in_file_properties", "True")
+
         newbranch = newdir.import_branch(bzrwt.branch)
 
         c = ra.RemoteAccess(repos_url)
@@ -559,41 +552,40 @@ class RepositoryTests(SubversionTestCase):
         revid2 = bzrwt.commit("Merge something", rev_id="mergerevid")
         bzr_parents = bzrwt.branch.repository.get_revision(revid2).parent_ids
         trunk = Branch.open(repos_url + "/trunk")
-        self.assertRaises(AppendRevisionsOnlyViolation, 
+        self.assertRaises(AppendRevisionsOnlyViolation,
             trunk.pull, bzrwt.branch)
-        trunk.get_config().set_user_option('append_revisions_only', 'False')
+        trunk.set_append_revisions_only(False)
         trunk.pull(bzrwt.branch)
 
-        self.assertEquals(tuple(bzr_parents), 
+        self.assertEquals(tuple(bzr_parents),
                 trunk.repository.get_revision(revid2).parent_ids)
 
-        self.assertEquals([revid1, revid2], trunk.revision_history())
+        self.assertEquals((2, revid2), trunk.last_revision_info())
         self.assertEquals(
                 '1 initialrevid\n2 mergerevid\n',
                 self.client_get_prop(repos_url+"/trunk", SVN_PROP_BZR_REVISION_ID+"v3-trunk0",
                                      c.get_latest_revnum()))
 
     def test_revision_history(self):
-        repos_url = self.make_repository('a')
+        repos_url = self.make_svn_repository('a')
 
         branch = Branch.open(repos_url)
-        self.assertEqual([branch.generate_revision_id(0)], 
-                branch.revision_history())
+        self.assertEqual((1, branch.generate_revision_id(0)),
+                branch.last_revision_info())
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
         dc.change_prop(SVN_PROP_BZR_REVISION_ID+"v3-none", 
                 "42 mycommit\n")
         dc.close()
-        
+
         branch = Branch.open(repos_url)
         repos = Repository.open(repos_url)
-        
+
         mapping = repos.get_mapping()
 
-        self.assertEqual([repos.generate_revision_id(0, "", mapping), 
-                    repos.generate_revision_id(1, "", mapping)], 
-                branch.revision_history())
+        self.assertEqual((42, repos.generate_revision_id(1, "", mapping)),
+                branch.last_revision_info())
 
         dc = self.get_commit_editor(repos_url)
         dc.open_file("foo").modify()
@@ -604,11 +596,9 @@ class RepositoryTests(SubversionTestCase):
 
         mapping = repos.get_mapping()
 
-        self.assertEqual([
-            repos.generate_revision_id(0, "", mapping),
-            "mycommit",
-            repos.generate_revision_id(2, "", mapping)],
-            branch.revision_history())
+        self.assertEqual(
+            (43, repos.generate_revision_id(2, "", mapping)),
+            branch.last_revision_info())
 
 
 class ErrorTests(TestCase):
