@@ -18,12 +18,20 @@
 import os
 import sys
 
+from bzrlib.controldir import (
+    ControlDir,
+    format_registry,
+    )
 import bzrlib.gpg
+from bzrlib import version_info as bzrlib_version
 from bzrlib.repository import Repository
+from bzrlib.tests import KnownFailure, TestSkipped
 from bzrlib.tests.blackbox import ExternalBase
 
 from bzrlib.plugins.svn.convert import load_dumpfile
-from bzrlib.plugins.svn.layout.standard import RootLayout
+from bzrlib.plugins.svn.layout.standard import (
+    RootLayout,
+    )
 from bzrlib.plugins.svn.tests import SubversionTestCase
 
 
@@ -31,22 +39,14 @@ class TestBranch(SubversionTestCase, ExternalBase):
 
     def setUp(self):
         ExternalBase.setUp(self)
-        if getattr(self, "_init_client", None) is not None:
-            self._init_client()
-        else:
-            # Subvertpy < 0.7.4 doesn't provide _init_client.
-            from subvertpy import client, ra
-            self.client_ctx = client.Client()
-            self.client_ctx.auth = ra.Auth([ra.get_simple_provider(),
-                                            ra.get_username_provider(), ])
-            self.client_ctx.log_msg_func = self.log_message_func
+        self._init_client()
 
     def tearDown(self):
         ExternalBase.tearDown(self)
 
     def test_branch_empty(self):
-        repos_url = self.make_repository('d')
-        self.run_bzr("branch %s dc" % repos_url)
+        branch = self.make_svn_branch('d')
+        self.run_bzr("branch %s dc" % branch.base)
 
     def commit_something(self, repos_url):
         dc = self.get_commit_editor(repos_url)
@@ -54,28 +54,33 @@ class TestBranch(SubversionTestCase, ExternalBase):
         dc.close()
 
     def test_branch_onerev(self):
-        repos_url = self.make_client('d', 'de')
-        self.commit_something(repos_url)
-        self.run_bzr("branch %s dc" % repos_url)
-        self.assertEquals("2\n", self.run_bzr("revno de")[0])
+        tree = self.make_svn_branch_and_tree('d', 'de')
+        self.commit_something(tree.branch.base)
+        self.run_bzr("branch %s dc" % tree.branch.base)
+        self.assertEquals("1\n", self.run_bzr("revno de")[0])
 
     def test_branch_onerev_stacked(self):
-        repos_url = self.make_client('d', 'de')
-        self.commit_something(repos_url)
-        self.run_bzr("branch --stacked %s dc" % repos_url, retcode=3)
+        tree = self.make_svn_branch_and_tree('d', 'de')
+        self.commit_something(tree.branch.base)
+        self.run_bzr("branch --stacked %s dc" % tree.branch.base, retcode=3)
 
     def test_log_empty(self):
-        repos_url = self.make_repository('d')
-        self.run_bzr('log %s' % repos_url)
+        branch = self.make_svn_branch('d')
+        self.run_bzr('log %s' % branch.base)
 
     def test_info_verbose(self):
-        repos_url = self.make_repository('d')
-        self.run_bzr('info -v %s' % repos_url)
+        branch = self.make_svn_branch('d')
+        self.run_bzr('info -v %s' % branch.base)
 
     def test_pack(self):
         repos_url = self.make_repository('d')
         self.run_bzr('pack %s' % repos_url)
 
+    def test_rmbranch(self):
+        branch = self.make_svn_branch('d')
+
+        self.run_bzr("rmbranch %s" % branch.base)
+
     def test_push_create_prefix(self):
         repos_url = self.make_repository('d')
 
@@ -152,21 +157,20 @@ class TestBranch(SubversionTestCase, ExternalBase):
         self.assertContainsRe(output, "Reconciliation complete.\n")
 
     def test_missing(self):
-        repos_url = self.make_repository('d')
+        branch = self.make_svn_branch('d')
 
         self.run_bzr("init dc")
 
         os.chdir("dc")
-        output, err = self.run_bzr("missing %s" % repos_url, retcode=1)
-        self.assertContainsRe(output, "You are missing 1 revision\\(s\\):")
+        output, err = self.run_bzr("missing %s" % branch.base)
+        self.assertContainsRe(output, "Branches are up to date.")
 
-        dc = self.get_commit_editor(repos_url)
-        trunk = dc.add_dir('trunk')
-        trunk.add_file("trunk/foo").modify()
+        dc = self.get_commit_editor(branch.base)
+        dc.add_file("foo").modify()
         dc.close()
 
-        output, err = self.run_bzr("missing %s" % repos_url, retcode=1)
-        self.assertContainsRe(output, "You are missing 2 revision\\(s\\):")
+        output, err = self.run_bzr("missing %s" % branch.base, retcode=1)
+        self.assertContainsRe(output, "You are missing 1 revision")
 
     def test_push_overwrite(self):
         repos_url = self.make_repository('d')
@@ -185,12 +189,14 @@ class TestBranch(SubversionTestCase, ExternalBase):
 
     def test_dpush_empty(self):
         repos_url = self.make_repository('dp')
+        Repository.open(repos_url).store_layout(RootLayout())
         self.run_bzr("init --default-rich-root dc")
         os.chdir("dc")
         self.run_bzr("dpush %s" % repos_url)
 
     def test_dpush(self):
         repos_url = self.make_repository('d')
+        Repository.open(repos_url).store_layout(RootLayout())
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
@@ -204,6 +210,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
 
     def test_dpush_new(self):
         repos_url = self.make_repository('d')
+        Repository.open(repos_url).store_layout(RootLayout())
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
@@ -219,6 +226,7 @@ class TestBranch(SubversionTestCase, ExternalBase):
 
     def test_dpush_wt_diff(self):
         repos_url = self.make_repository('d')
+        Repository.open(repos_url).store_layout(RootLayout())
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("foo").modify()
@@ -234,7 +242,11 @@ class TestBranch(SubversionTestCase, ExternalBase):
                 self.run_bzr("status dc")[0])
 
     def test_info_workingtree(self):
-        repos_url = self.make_client('d', 'dc')
+        self.make_svn_branch_and_tree('d', 'dc')
+        # Info is broken against bzr 2.3
+        if bzrlib_version < (2, 4, 0):
+            raise KnownFailure("bzr info against svn repositories is "
+                "known broken against 2.3")
         self.run_bzr('info -v dc')
 
     def test_dumpfile(self):
@@ -399,11 +411,11 @@ Node-copyfrom-path: x
             mapping.revision_id_foreign_to_bzr((uuid, "", 5))))
         self.assertTrue(newrepos.has_revision(
             mapping.revision_id_foreign_to_bzr((uuid, "", 1))))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 mapping.revision_id_foreign_to_bzr((uuid, "", 1)))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 mapping.revision_id_foreign_to_bzr((uuid, "", 5)))
-        self.assertNotEqual(inv1.path2id("y"), inv2.path2id("y"))
+        self.assertNotEqual(tree1.path2id("y"), tree2.path2id("y"))
 
     def test_svn_import_bzr_branch(self):
         self.run_bzr('init foo')
@@ -435,7 +447,7 @@ Node-copyfrom-path: x
                 self.run_bzr('info a')[0])
 
     def test_lightweight_checkout_lightweight_checkout(self):
-        repos_url = self.make_client("a", "dc")
+        self.make_svn_branch_and_tree("a", "dc")
         self.build_tree({'dc/foo': "test", 'dc/bla': "ha"})
         self.client_add("dc/foo")
         self.client_add("dc/bla")
@@ -443,11 +455,11 @@ Node-copyfrom-path: x
         self.run_bzr("checkout --lightweight dc de")
 
     def test_commit(self):
-        repos_url = self.make_client('d', 'de')
+        self.make_svn_branch_and_tree('d', 'de')
         self.build_tree({'de/foo': 'bla'})
         self.run_bzr("add de/foo")
         self.run_bzr("commit -m test de")
-        self.assertEquals("2\n", self.run_bzr("revno de")[0])
+        self.assertEquals("1\n", self.run_bzr("revno de")[0])
 
     # this method imported from bzrlib.tests.test_msgeditor:
     def make_fake_editor(self, message='test message from fed\\n'):
@@ -578,6 +590,12 @@ if len(sys.argv) == 2:
         os.chdir('bzr-trunk')
         self.run_bzr('merge %s/branches/my-branch' % svn_url)
         self.run_bzr('nick my-branch')
+        f = open('.bzr/branch/branch.conf', 'w')
+        try:
+            f.write("append_revisions_only = False\n")
+        finally:
+            f.close()
+
         self.run_bzr('ci -m "Merge my-branch"')
 
         os.chdir(cwd)
@@ -588,7 +606,10 @@ if len(sys.argv) == 2:
         svn_url = self.make_repository('d')
 
         self.run_bzr('svn-import --format 1.9-rich-root %s dc' % svn_url)
-        self.assertEquals("Repository branch (format: 1.14-rich-root)\nLocation:\n  shared repository: dc\n  repository branch: dc\n\nRelated branches:\n  parent branch: d\n", self.run_bzr('info dc')[0])
+        cd = ControlDir.open('dc')
+        self.assertEquals(
+            cd.open_repository()._format,
+            format_registry.make_bzrdir('1.9-rich-root').repository_format)
 
     def test_svn_layout(self):
         svn_url = self.make_repository('d')
@@ -597,5 +618,80 @@ if len(sys.argv) == 2:
         dc.add_dir("trunk")
         dc.close()
 
-        self.assertEquals('Repository root: %s\nLayout: root\nBranch path: \nPush merged revisions: False\n' % svn_url,
+        self.assertEquals(
+            'Repository root: %s\n'
+            'Layout: trunk-variable\n'
+            'Branch path: trunk\n'
+            'Tag container directory: tags\n'
+            'Branch container directory: branches\n'
+            'Push merged revisions: None\n' % svn_url,
                 self.run_bzr('svn-layout %s' % svn_url)[0])
+
+    def test_svn_branches(self):
+        svn_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(svn_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        dc = self.get_commit_editor(svn_url)
+        branches = dc.add_dir("branches")
+        branches.add_dir("branches/somebranch", "trunk")
+        tags = dc.add_dir("tags")
+        tags.add_dir("tags/release-1.0", "trunk")
+        dc.close()
+
+        self.assertEquals(
+            'Branches:\n'
+            'branches/somebranch (somebranch)\n'
+            'trunk (trunk)\n'
+            'Tags:\n'
+            'tags/release-1.0 (release-1.0)\n',
+            self.run_bzr('svn-branches --layout trunk %s' % svn_url)[0])
+
+    def test_diff(self):
+        self.make_svn_branch_and_tree('d', 'dc')
+        self.build_tree_contents([("dc/file", "bar")])
+        self.client_add("dc/file")
+        self.run_bzr('diff dc', retcode=True)
+
+    def test_diff_svn(self):
+        self.make_svn_branch_and_tree('d', 'dc')
+        self.build_tree_contents([("dc/file", "bar")])
+        self.client_add("dc/file")
+        self.run_bzr('diff --format=svn dc', retcode=True)
+
+    def test_ls(self):
+        self.make_svn_branch_and_tree('d', 'dc')
+        self.build_tree_contents([("dc/file", "bar")])
+        self.client_add("dc/file")
+        self.assertEquals("dc/file\n",
+            self.run_bzr('ls dc', retcode=0)[0])
+
+    def test_log_v(self):
+        tree = self.make_svn_branch_and_tree('d', 'dc')
+        self.commit_something(tree.branch.base)
+        self.run_bzr('log -v d', retcode=0)
+        self.run_bzr('log -v dc', retcode=0)
+
+    def test_svn_import_colocated(self):
+        if bzrlib_version < (2, 5, 0):
+            raise TestSkipped("Colocated branches not supported in bzr < 2.5")
+
+        svn_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(svn_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        dc = self.get_commit_editor(svn_url)
+        branches = dc.add_dir("branches")
+        branches.add_dir("branches/somebranch", "trunk")
+        tags = dc.add_dir("tags")
+        tags.add_dir("tags/release-1.0", "trunk")
+        dc.close()
+
+        self.run_bzr('init --development-colo dc')
+        self.run_bzr('svn-import d dc')
+        cd = ControlDir.open('dc')
+        self.assertEquals(set([None, "somebranch"]), set([b.name for b in cd.list_branches()]))
