@@ -18,9 +18,12 @@
 
 """Commit and push tests."""
 
-from copy import copy
 import os
-from subvertpy import ra
+from subvertpy import (
+    ERR_FS_TXN_OUT_OF_DATE,
+    SubversionException,
+    ra,
+    )
 from subvertpy.properties import time_to_cstring
 
 from bzrlib.branch import (
@@ -29,161 +32,209 @@ from bzrlib.branch import (
     )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import (
-    DivergedBranches,
     BzrError,
+    DivergedBranches,
+    NoSuchId,
+    )
+from bzrlib.inventory import (
+    InventoryFile,
+    InventoryLink,
     )
 from bzrlib.repository import Repository
 from bzrlib.tests import (
     TestCase,
     TestSkipped,
     )
+try:
+    from bzrlib.tests.features import (
+        UnicodeFilenameFeature
+        )
+except ImportError: # bzr < 2.5
+    from bzrlib.tests import (
+        UnicodeFilenameFeature
+        )
 from bzrlib.trace import mutter
 from bzrlib.workingtree import WorkingTree
 
-from bzrlib.plugins.svn import format
 from bzrlib.plugins.svn.commit import (
+    file_editor_send_prop_changes,
     set_svn_revprops,
     _revision_id_to_svk_feature,
     )
-from bzrlib.plugins.svn.errors import RevpropChangeFailed
+from bzrlib.plugins.svn.errors import (
+    RevpropChangeFailed,
+    )
 from bzrlib.plugins.svn.mapping import mapping_registry
 from bzrlib.plugins.svn.tests import SubversionTestCase
 
+
 class TestNativeCommit(SubversionTestCase):
 
     def test_simple_commit(self):
-        self.make_client('d', 'dc')
+        wt = self.make_svn_branch_and_tree('d', 'dc') #1
         self.build_tree({'dc/foo/bla': "data"})
         self.client_add("dc/foo")
-        wt = WorkingTree.open("dc")
         revid = wt.commit(message="data")
-        self.assertEqual(wt.branch.generate_revision_id(1), revid)
+        self.assertEqual(wt.branch.generate_revision_id(2), revid)
         self.client_update("dc")
-        self.assertEqual(wt.branch.generate_revision_id(1),
+        self.assertEqual(wt.branch.generate_revision_id(2),
                 wt.branch.last_revision())
         wt = WorkingTree.open("dc")
-        new_inventory = wt.branch.repository.get_inventory(
+        new_tree = wt.branch.repository.revision_tree(
                             wt.branch.last_revision())
-        self.assertTrue(new_inventory.has_filename("foo"))
-        self.assertTrue(new_inventory.has_filename("foo/bla"))
+        self.assertTrue(new_tree.has_filename("foo"))
+        self.assertTrue(new_tree.has_filename("foo/bla"))
 
     def test_commit_message(self):
-        self.make_client('d', 'dc')
+        wt = self.make_svn_branch_and_tree('d', 'dc') #1
         self.build_tree({'dc/foo/bla': "data"})
         self.client_add("dc/foo")
-        wt = WorkingTree.open("dc")
-        revid = wt.commit(message="data")
-        self.assertEqual(wt.branch.generate_revision_id(1), revid)
+        revid = wt.commit(message="data") #2
+        self.assertEqual(wt.branch.generate_revision_id(2), revid)
         self.assertEqual(
-                wt.branch.generate_revision_id(1), wt.branch.last_revision())
+                wt.branch.generate_revision_id(2), wt.branch.last_revision())
         new_revision = wt.branch.repository.get_revision(
                             wt.branch.last_revision())
         self.assertEqual(wt.branch.last_revision(), new_revision.revision_id)
         self.assertEqual("data", new_revision.message)
 
     def test_commit_rev_id(self):
-        self.make_client('d', 'dc')
+        wt = self.make_svn_branch_and_tree('d', 'dc') #1
         self.build_tree({'dc/foo/bla': "data"})
         self.client_add("dc/foo")
-        wt = WorkingTree.open("dc")
-        revid = wt.commit(message="data", rev_id="some-revid-bla")
+        revid = wt.commit(message="data", rev_id="some-revid-bla") #2
         self.assertEqual("some-revid-bla", revid)
-        self.assertEqual(wt.branch.generate_revision_id(1), revid)
+        self.assertEqual(wt.branch.generate_revision_id(2), revid)
         self.assertEqual(
-                wt.branch.generate_revision_id(1), wt.branch.last_revision())
+                wt.branch.generate_revision_id(2), wt.branch.last_revision())
         new_revision = wt.branch.repository.get_revision(
                             wt.branch.last_revision())
         self.assertEqual(wt.branch.last_revision(), new_revision.revision_id)
 
     def test_commit_unicode_filename(self):
-        self.make_client('d', 'dc')
-        try:
-            self.build_tree({u'dc/I²C': "data"})
-        except UnicodeError:
-            raise TestSkipped("This platform does not support unicode symlinks")
+        self.requireFeature(UnicodeFilenameFeature)
+        wt = self.make_svn_branch_and_tree('d', 'dc')
+        self.build_tree({u'dc/I²C': "data"})
         self.client_add(u"dc/I²C".encode("utf-8"))
-        wt = WorkingTree.open("dc")
         wt.commit(message="data")
 
-
     def test_commit_local(self):
-        self.make_client('d', 'dc')
+        wt = self.make_svn_branch_and_tree('d', 'dc')
         self.build_tree({'dc/foo/bla': "data"})
         self.client_add("dc/foo")
-        wt = WorkingTree.open("dc")
         self.assertRaises(BzrError, wt.commit,
                 message="data", local=True)
 
     def test_commit_committer(self):
-        self.make_client('d', 'dc')
+        wt = self.make_svn_branch_and_tree('d', 'dc')
         self.build_tree({'dc/foo/bla': "data"})
         self.client_add("dc/foo")
-        wt = WorkingTree.open("dc")
         revid = wt.commit(message="data", committer="john doe")
         rev = wt.branch.repository.get_revision(revid)
         self.assertEquals("john doe", rev.committer)
 
     def test_commit_message_nordic(self):
-        self.make_client('d', 'dc')
+        wt = self.make_svn_branch_and_tree('d', 'dc') #1
         self.build_tree({'dc/foo/bla': "data"})
         self.client_add("dc/foo")
-        wt = WorkingTree.open("dc")
-        revid = wt.commit(message=u"føø")
-        self.assertEqual(revid, wt.branch.generate_revision_id(1))
+        revid = wt.commit(message=u"føø") #2
+        self.assertEqual(revid, wt.branch.generate_revision_id(2))
         self.assertEqual(
-                wt.branch.generate_revision_id(1), wt.branch.last_revision())
+                wt.branch.generate_revision_id(2), wt.branch.last_revision())
         new_revision = wt.branch.repository.get_revision(
                             wt.branch.last_revision())
         self.assertEqual(wt.branch.last_revision(), new_revision.revision_id)
         self.assertEqual(u"føø", new_revision.message)
 
+    def test_commit_out_of_date(self):
+        branch = self.make_svn_branch('d')
+
+        dc = self.get_commit_editor(branch.base)
+        somedir = dc.add_dir("somedir")
+        somefile = somedir.add_file("somedir/somefile").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(branch.repository.base)
+
+        br = Branch.open(branch.base)
+        br.lock_write()
+        self.addCleanup(br.unlock)
+        br.set_append_revisions_only(False)
+        builder = br.get_commit_builder([br.last_revision()])
+        builder.finish_inventory()
+
+        trunk = dc.open_dir("trunk")
+        somedir = trunk.open_dir("trunk/somedir")
+        somefile = somedir.open_file("trunk/somedir/somefile").modify()
+        dc.close()
+
+        e = self.assertRaises(SubversionException, builder.commit, "foo")
+
+        self.assertEquals(ERR_FS_TXN_OUT_OF_DATE, e.args[1])
+
+    def test_lossy_commit_builder(self):
+        branch = self.make_svn_branch('d', lossy=True) #1
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
+        cb = branch.repository.get_commit_builder(branch,
+            [branch.last_revision()], branch.get_config(),
+            revision_id="foorevid", lossy=True)
+        revid = cb.commit("msg")
+        self.assertEquals(
+            branch.repository.generate_revision_id(2, branch.get_branch_path(), branch.mapping),
+            revid)
+
     def test_commit_update(self):
-        self.make_client('d', 'dc')
+        self.make_svn_branch_and_tree('d', 'dc') #1
         self.build_tree({'dc/foo/bla': "data"})
         self.client_add("dc/foo")
+        self.client_commit("dc", "msg") #2
         wt = WorkingTree.open("dc")
+        self.build_tree({'dc/foo/bla': "data2"})
         wt.set_pending_merges(["some-ghost-revision"])
-        wt.commit(message="data")
+        wt.commit(message="data") #3
         self.assertEqual(
-                wt.branch.generate_revision_id(1),
+                wt.branch.generate_revision_id(3),
                 wt.branch.last_revision())
 
     def test_commit_rename_file(self):
-        repos_url = self.make_client('d', 'dc')
+        wt = self.make_svn_branch_and_tree('d', 'dc') #1
+        self.build_tree({'dc/.dummy': 'dummy'})
+        self.client_add('dc/.dummy')
+        self.client_commit("dc", "msg") #2
         self.build_tree({'dc/foo': "data"})
         self.client_add("dc/foo")
-        wt = WorkingTree.open("dc")
         wt.set_pending_merges(["some-ghost-revision"])
         oldid = wt.path2id("foo")
-        wt.commit(message="data") # 1
+        wt.commit(message="data") # 3
         wt.rename_one("foo", "bar")
         self.assertEquals(oldid, wt.path2id("bar"))
         self.assertEquals(None, wt.path2id("foo"))
-        wt.commit(message="doe") # 2
-        paths = self.client_log(repos_url, 2, 0)[2][0]
-        self.assertEquals('D', paths["/foo"][0])
-        self.assertEquals('A', paths["/bar"][0])
-        self.assertEquals('/foo', paths["/bar"][1])
-        self.assertEquals(1, paths["/bar"][2])
-        svnrepo = Repository.open(repos_url)
+        wt.commit(message="doe") # 4
+        paths = self.client_log(wt.branch.base, 4, 0)[4][0]
+        self.assertEquals('D', paths["/trunk/foo"][0])
+        self.assertEquals('A', paths["/trunk/bar"][0])
+        self.assertEquals('/trunk/foo', paths["/trunk/bar"][1])
+        self.assertEquals(3, paths["/trunk/bar"][2])
+        svnrepo = Repository.open(wt.branch.repository.base)
+        revmeta = svnrepo._revmeta_provider.get_revision(wt.branch.get_branch_path(), 4)
         self.assertEquals({u"bar": oldid},
-                svnrepo._revmeta_provider.get_revision("", 2).get_fileid_overrides(svnrepo.get_mapping()))
+                revmeta.get_fileid_overrides(svnrepo.get_mapping()))
 
     def test_commit_rename_file_from_directory(self):
-        repos_url = self.make_client('d', 'dc')
+        wt = self.make_svn_branch_and_tree('d', 'dc') #1
         self.build_tree({'dc/adir/foo': "data"})
         self.client_add("dc/adir")
-        wt = WorkingTree.open("dc")
-        wt.commit(message="data")
+        wt.commit(message="data") #2
         wt.rename_one("adir/foo", "bar")
         self.assertFalse(wt.has_filename("adir/foo"))
         self.assertTrue(wt.has_filename("bar"))
-        wt.commit(message="doe")
-        paths = self.client_log(repos_url, 2, 0)[2][0]
-        self.assertEquals('D', paths["/adir/foo"][0])
-        self.assertEquals('A', paths["/bar"][0])
-        self.assertEquals('/adir/foo', paths["/bar"][1])
-        self.assertEquals(1, paths["/bar"][2])
+        wt.commit(message="doe") #3
+        paths = self.client_log(wt.branch.base, 3, 0)[3][0]
+        self.assertEquals('D', paths["/trunk/adir/foo"][0])
+        self.assertEquals('A', paths["/trunk/bar"][0])
+        self.assertEquals('/trunk/adir/foo', paths["/trunk/bar"][1])
+        self.assertEquals(2, paths["/trunk/bar"][2])
 
     def test_commit_sets_mergeinfo(self):
         repos_url = self.make_repository('d')
@@ -208,14 +259,11 @@ class TestNativeCommit(SubversionTestCase):
 
         branch = Branch.open(repos_url+"/trunk")
         foobranch = Branch.open(repos_url+"/tags/foo")
-        builder = branch.get_commit_builder([branch.last_revision(), foobranch.last_revision()],
-                revision_id="my-revision-id")
-        tree = branch.repository.revision_tree(branch.last_revision())
-        new_tree = copy(tree)
-        ie = new_tree.inventory.root
-        ie.revision = None
-        builder.record_entry_contents(ie, [tree.inventory], '', new_tree,
-                                      None)
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
+        builder = branch.get_commit_builder(
+            [branch.last_revision(), foobranch.last_revision()],
+            revision_id="my-revision-id")
         builder.finish_inventory()
         builder.commit("foo")
 
@@ -228,7 +276,7 @@ class TestNativeCommit(SubversionTestCase):
             mi = c.mergeinfo(["trunk"], 4)
             self.assertEquals({"trunk": {"/tags/foo": [(1, 3, 1)]}}, mi)
         except NotImplementedError:
-            pass # Svn 1.4
+            raise TestSkipped("mergeinfo not available with svn 1.4")
 
     def test_mwh(self):
         repo = self.make_client('d', 'sc')
@@ -266,6 +314,23 @@ class TestNativeCommit(SubversionTestCase):
         self.client_update('pyd')
         self.assertEqual("data", open('pyd/bar').read())
 
+    def test_parent_id_reused(self):
+        wt = self.make_svn_branch_and_tree('d', 'dc')
+
+        first_revid = wt.commit("msg", rev_id="firstrevid")
+        second_revid = wt.commit("msg", rev_id="secondrevid")
+
+        wt.branch.set_last_revision_info(1, first_revid)
+
+        branch = wt.branch
+
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
+        cb = branch.get_commit_builder([branch.last_revision()],
+            revision_id="foorevid")
+        revid = cb.commit("msg")
+        self.assertEquals("foorevid", revid)
+
 
 class TestPush(SubversionTestCase):
 
@@ -282,14 +347,16 @@ class TestPush(SubversionTestCase):
         self.newdir = self.olddir.sprout("dc")
 
     def test_empty(self):
-        self.assertEqual(0, int(self.olddir.open_branch().pull(
-                                self.newdir.open_branch())))
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEquals(2, result.old_revno)
+        self.assertEquals(2, result.new_revno)
 
     def test_empty_result(self):
-        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        target_branch = self.olddir.open_branch()
+        result = target_branch.pull(self.newdir.open_branch())
         self.assertIsInstance(result, PullResult)
         self.assertEqual(result.old_revno, self.olddir.open_branch().revno())
-        self.assertEqual(result.master_branch, None)
+        self.assertEqual(result.master_branch, target_branch)
         self.assertEqual(result.source_branch.bzrdir.root_transport.base,
                          self.newdir.root_transport.base)
 
@@ -298,8 +365,9 @@ class TestPush(SubversionTestCase):
         self.client_add("sc/foo/bar")
         self.client_commit("sc", "second message")
 
-        self.assertEqual(0, int(self.olddir.open_branch().pull(
-                                self.newdir.open_branch())))
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEquals(3, result.old_revno)
+        self.assertEquals(3, result.new_revno)
 
     def test_diverged(self):
         self.build_tree({'sc/foo/bar': "data"})
@@ -318,28 +386,42 @@ class TestPush(SubversionTestCase):
                           self.newdir.open_branch())
 
     def test_unicode_filename(self):
-        try:
-            self.build_tree({u'dc/I²C': 'other data'})
-        except UnicodeError:
-            raise TestSkipped("This platform does not support unicode paths")
+        self.requireFeature(UnicodeFilenameFeature)
+        self.build_tree({u'dc/I²C': 'other data'})
         wt = self.newdir.open_workingtree()
         wt.add(u'I²C')
         wt.commit(message="Commit from Bzr")
-        self.assertEqual(1, int(self.olddir.open_branch().pull(
-                                self.newdir.open_branch())))
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEqual(2, result.old_revno)
+        self.assertEquals(3, result.new_revno)
+
+    def test_add_file_to_unicode_folder(self):
+        self.requireFeature(UnicodeFilenameFeature)
+        self.build_tree({u'dc/I²C/': None})
+        wt = self.newdir.open_workingtree()
+        wt.add(u'I²C')
+        wt.commit(message="Commit from Bzr")
+        self.build_tree({u'dc/I²C/file': 'data'})
+        wt.add(u'I²C/file')
+        wt.commit(message="Commit from Bzr2")
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEquals(2, result.old_revno)
+        self.assertEquals(4, result.new_revno)
+        owt = self.olddir.open_workingtree()
+        owt.update()
+        self.assertTrue(owt.has_filename(u'I²C/file'))
 
     def test_rename_from_unicode_filename(self):
-        try:
-            self.build_tree({u'dc/I²C': 'other data'})
-        except UnicodeError:
-            raise TestSkipped("This platform does not support symlink paths")
+        self.requireFeature(UnicodeFilenameFeature)
+        self.build_tree({u'dc/I²C': 'other data'})
         wt = self.newdir.open_workingtree()
         wt.add(u'I²C')
         wt.commit(message="Commit from Bzr")
         wt.rename_one(u'I²C', u'I²C2')
         wt.commit(message="Commit from Bzr2")
-        self.assertEqual(2, int(self.olddir.open_branch().pull(
-                                self.newdir.open_branch())))
+        result = self.olddir.open_branch().pull(self.newdir.open_branch())
+        self.assertEquals(2, result.old_revno)
+        self.assertEquals(4, result.new_revno)
 
     def test_change(self):
         self.build_tree({'dc/foo/bla': 'other data'})
@@ -350,15 +432,15 @@ class TestPush(SubversionTestCase):
 
         repos = self.olddir._find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
-                         inv[inv.path2id('foo/bla')].revision)
+                         tree.get_file_revision(tree.path2id('foo/bla')))
         self.assertEqual(wt.branch.last_revision(),
           repos.generate_revision_id(2, "", mapping))
         self.assertEqual(wt.branch.last_revision(),
                         self.olddir.open_branch().last_revision())
         self.assertEqual("other data",
-            repos.revision_tree(repos.generate_revision_id(2, "", mapping)).get_file_text( inv.path2id("foo/bla")))
+            repos.revision_tree(repos.generate_revision_id(2, "", mapping)).get_file_text(tree.path2id("foo/bla")))
 
     def test_simple(self):
         self.build_tree({'dc/file': 'data'})
@@ -370,8 +452,8 @@ class TestPush(SubversionTestCase):
 
         repos = self.olddir._find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv.has_filename('file'))
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertTrue(tree.has_filename('file'))
         self.assertEqual(wt.branch.last_revision(),
                 repos.generate_revision_id(2, "", mapping))
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
@@ -387,8 +469,8 @@ class TestPush(SubversionTestCase):
 
         repos = self.olddir._find_repository()
         mapping = repos.get_mapping()
-        inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv.has_filename('file'))
+        tree = repos.revision_tree(repos.generate_revision_id(2, "", mapping))
+        self.assertTrue(tree.has_filename('file'))
         self.assertEquals(wt.branch.last_revision(),
                          repos.generate_revision_id(2, "", mapping))
 
@@ -533,7 +615,6 @@ class JoinedCommitTests(SubversionTestCase):
 
         self.olddir.open_branch().pull(self.newdir.open_branch())
         paths = self.client_log(repos_url, 4, 0)[4][0]
-        mutter('paths %r' % paths)
         self.assertEquals(('A', "/branches/newbranch", 2), paths["/trunk/newdir"])
 
 
@@ -566,8 +647,7 @@ class TestPushNested(SubversionTestCase):
 class HeavyWeightCheckoutTests(SubversionTestCase):
 
     def test_bind(self):
-        repos_url = self.make_repository("d")
-        master_branch = Branch.open(repos_url)
+        master_branch = self.make_svn_branch("d")
         os.mkdir("b")
         local_dir = master_branch.bzrdir.sprout("b")
         wt = local_dir.open_workingtree()
@@ -575,8 +655,7 @@ class HeavyWeightCheckoutTests(SubversionTestCase):
         local_dir.open_branch().unbind()
 
     def test_commit(self):
-        repos_url = self.make_repository("d")
-        master_branch = Branch.open(repos_url)
+        master_branch = self.make_svn_branch("d")
         os.mkdir("b")
         local_dir = master_branch.bzrdir.sprout("b")
         wt = local_dir.open_workingtree()
@@ -585,41 +664,42 @@ class HeavyWeightCheckoutTests(SubversionTestCase):
         wt.branch.nick = "somenick"
         wt.add('file')
         revid = wt.commit(message="Commit from Bzr")
-        master_branch = Branch.open(repos_url)
+        master_branch = Branch.open(master_branch.base)
         self.assertEquals(revid, master_branch.last_revision())
 
     def test_fileid(self):
-        repos_url = self.make_repository("d")
-        master_branch = Branch.open(repos_url)
+        master_branch = self.make_svn_branch("d")
         os.mkdir("b")
         local_dir = master_branch.bzrdir.sprout("b")
         wt = local_dir.open_workingtree()
         local_dir.open_branch().bind(master_branch)
         self.build_tree({'b/file': 'data'})
+        rootid = wt.get_root_id()
         wt.branch.nick = "some-nick"
         wt.add('file')
         oldid = wt.path2id("file")
         revid1 = wt.commit(message="Commit from Bzr")
         wt.rename_one('file', 'file2')
         revid2 = wt.commit(message="Commit from Bzr")
-        master_branch = Branch.open(repos_url)
+        master_branch = Branch.open(master_branch.base)
         rm_provider = master_branch.repository._revmeta_provider
         mapping = master_branch.repository.get_mapping()
-        self.assertEquals({u"file": oldid},
-                rm_provider.get_revision("", 1).get_fileid_overrides(mapping))
+        revmeta1 = rm_provider.get_revision(master_branch.get_branch_path(), 2)
+        self.assertEquals({u'': rootid, u"file": oldid},
+                revmeta1.get_fileid_overrides(mapping))
+        revmeta2 = rm_provider.get_revision(master_branch.get_branch_path(), 3)
         self.assertEquals({u"file2": oldid},
-                rm_provider.get_revision("", 2).get_fileid_overrides(mapping))
+                revmeta2.get_fileid_overrides(mapping))
         tree1 = master_branch.repository.revision_tree(revid1)
         tree2 = master_branch.repository.revision_tree(revid2)
         delta = tree2.changes_from(tree1)
-        mutter("changes %r" % list(rm_provider.iter_reverse_branch_changes("", 2, 0)))
+        mutter("changes %r" % list(rm_provider.iter_reverse_branch_changes(master_branch.get_branch_path(), 3, 0)))
         self.assertEquals(0, len(delta.added))
         self.assertEquals(0, len(delta.removed))
         self.assertEquals(1, len(delta.renamed))
 
     def test_nested_fileid(self):
-        repos_url = self.make_repository("d")
-        master_branch = Branch.open(repos_url)
+        master_branch = self.make_svn_branch("d") #1
         os.mkdir("b")
         local_dir = master_branch.bzrdir.sprout("b")
         wt = local_dir.open_workingtree()
@@ -628,15 +708,58 @@ class HeavyWeightCheckoutTests(SubversionTestCase):
         self.build_tree({'b/dir/file': 'data'})
         wt.add('dir')
         wt.add('dir/file')
+        rootid = wt.get_root_id()
         dirid = wt.path2id("dir")
         fileid = wt.path2id("dir/file")
-        revid1 = wt.commit(message="Commit from Bzr")
-        master_branch = Branch.open(repos_url)
+        revid1 = wt.commit(message="Commit from Bzr") #2
+        master_branch = Branch.open(master_branch.base)
         rm_provider = master_branch.repository._revmeta_provider
         mapping = master_branch.repository.get_mapping()
-        self.assertEquals({"dir": dirid,
-                          "dir/file": fileid},
-                          rm_provider.get_revision("", 1).get_fileid_overrides(mapping))
+        revmeta = rm_provider.get_revision(master_branch.get_branch_path(), 2)
+        self.assertEquals({
+            "": rootid,
+            "dir": dirid,
+            "dir/file": fileid},
+            revmeta.get_fileid_overrides(mapping))
+
+    def test_push_merged_revision(self):
+        repos_url = self.make_repository("d")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.add_dir("branches")
+        dc.add_dir("tags")
+        dc.close() # 1
+
+        master_branch = Branch.open(repos_url+"/trunk")
+        trunk = master_branch.bzrdir.sprout("trunk")
+        trunk.open_branch().bind(master_branch)
+
+        config = master_branch.repository.get_config()
+        config.set_user_option("push_merged_revisions", "True")
+
+        feature = trunk.sprout("feature")
+        wt = feature.open_workingtree()
+        self.build_tree_contents([("feature/test.txt", "test")])
+        wt.add("test.txt")
+        revid2 = wt.commit("added test.txt") # 2
+
+        wt = trunk.open_workingtree()
+        wt.merge_from_branch(feature.open_branch(), to_revision=revid2)
+        wt.commit("merged feature") # 3
+
+        log = self.client_log(repos_url, 3, 0)
+        self.assertEquals({
+            '/branches': ('A', None, -1),
+            '/tags': ('A', None, -1),
+            '/trunk': ('A', None, -1)}, log[1][0])
+        self.assertEquals({
+            '/branches/feature': ('A', '/trunk', 1),
+            '/branches/feature/test.txt': ('A', None, -1)}, log[2][0])
+        self.assertEquals({
+            '/trunk': ('M', None, -1),
+            '/trunk/test.txt': ('A', '/branches/feature/test.txt', 2)},
+            log[3][0])
 
 
 class RevpropTests(SubversionTestCase):
@@ -677,3 +800,66 @@ class SvkTestCase(TestCase):
                   mapping_registry.parse_revision_id))
 
 
+class SendPropChangesTests(TestCase):
+
+    class MockTree(object):
+
+        def __init__(self, file_id, kind, name, executable):
+            self._file_id = file_id
+            self._name = name
+            self._executable = executable
+            self._kind = kind
+
+        def has_id(self, file_id):
+            return (self._file_id == file_id)
+
+        def is_executable(self, file_id):
+            if self._file_id == file_id:
+                return self._executable
+            raise NoSuchId
+
+        def kind(self, file_id):
+            if self._file_id == file_id:
+                return self._kind
+            raise NoSuchId
+
+    class RecordingFileEditor(object):
+
+        def __init__(self):
+            self._props = {}
+
+        def change_prop(self, name, value):
+            self._props[name] = value
+
+    def test_become_symlink(self):
+        ie2 = InventoryLink("myfileid", "name", "theroot")
+        editor = SendPropChangesTests.RecordingFileEditor()
+        file_editor_send_prop_changes(
+            self.MockTree("myfileid", "file", "name", True), "myfileid",
+            ie2, editor)
+        self.assertEquals(editor._props,
+            {"svn:special": "*", "svn:executable": None})
+
+    def test_become_executable(self):
+        tree = self.MockTree("myfileid", "file", "name", False)
+        ie2 = InventoryFile("myfileid", "name", "theroot")
+        ie2.executable = True
+        editor = SendPropChangesTests.RecordingFileEditor()
+        file_editor_send_prop_changes(tree, "myfileid",ie2, editor)
+        self.assertEquals(editor._props, {"svn:executable": "*"})
+
+    def test_unbecome_executable(self):
+        tree = self.MockTree("myfileid", "file", "name", True)
+        ie2 = InventoryFile("myfileid", "name", "theroot")
+        ie2.executable = False
+        editor = SendPropChangesTests.RecordingFileEditor()
+        file_editor_send_prop_changes(tree, "myfileid", ie2, editor)
+        self.assertEquals(editor._props, {"svn:executable": None})
+
+    def test_nothing_changes(self):
+        tree = self.MockTree("myfileid", "file", "name", False)
+        ie2 = InventoryFile("myfileid", "name", "theroot")
+        editor = SendPropChangesTests.RecordingFileEditor()
+        file_editor_send_prop_changes(tree, "myfileid",ie2, editor)
+        self.assertEquals(editor._props, {})
+
