@@ -18,7 +18,6 @@
 
 """Subversion fetch tests."""
 
-
 import os
 import shutil
 import sys
@@ -32,7 +31,6 @@ from bzrlib.bzrdir import (
     BzrDir,
     )
 from bzrlib.osutils import (
-    has_symlinks,
     sha_string,
     )
 from bzrlib.repository import (
@@ -46,17 +44,19 @@ from bzrlib.tests import (
     TestCase,
     TestSkipped,
     )
+try:
+    from bzrlib.tests.features import (
+        SymlinkFeature,
+        )
+except ImportError: # bzr < 2.5
+    from bzrlib.tests import (
+        SymlinkFeature,
+        )
 from bzrlib.trace import (
     mutter,
     )
-from bzrlib.versionedfile import (
-    AbsentContentFactory,
-    ChunkedContentFactory,
-    FulltextContentFactory,
-    )
 
 from bzrlib.plugins.svn import (
-    format,
     remote,
     )
 from bzrlib.plugins.svn.convert import (
@@ -64,6 +64,7 @@ from bzrlib.plugins.svn.convert import (
     )
 from bzrlib.plugins.svn.errors import (
     InvalidFileName,
+    SymlinkTargetContainsNewline,
     )
 from bzrlib.plugins.svn.fetch import (
    FetchRevisionFinder,
@@ -75,6 +76,10 @@ from bzrlib.plugins.svn.layout.standard import (
     RootLayout,
     TrunkLayout,
     )
+from bzrlib.plugins.svn.mapping import (
+    SVN_REVPROP_BZR_MAPPING_VERSION,
+    SVN_REVPROP_BZR_ROOT,
+    )
 from bzrlib.plugins.svn.tests import (
     SubversionTestCase,
     )
@@ -100,7 +105,7 @@ class FetchTestCase(SubversionTestCase):
 class TestFetchWorks(FetchTestCase):
 
     def test_fetch_trunk1(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         proj1 = dc.add_dir("proj1")
@@ -115,7 +120,7 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
 
     def test_replace_from_branch(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         t = dc.add_dir("trunk")
@@ -175,7 +180,7 @@ class TestFetchWorks(FetchTestCase):
     def test_fetch_backslash(self):
         if sys.platform == 'win32':
             raise TestSkipped("Unable to create filenames with backslash on Windows")
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -190,7 +195,7 @@ class TestFetchWorks(FetchTestCase):
                 newrepos)
 
     def test_fetch_null(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
         oldrepos = Repository.open(repos_url)
         oldrepos.set_layout(TrunkLayout(1))
         dir = BzrDir.create("f")
@@ -221,7 +226,7 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
 
     def test_fetch_special_char(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -235,7 +240,7 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
 
     def test_fetch_signature(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
         trunk.add_file("trunk/bar").modify("data")
@@ -249,7 +254,7 @@ class TestFetchWorks(FetchTestCase):
         self.assertEquals("SIGNATURE", newrepos.get_signature_text(oldrepos.generate_revision_id(1, "trunk", oldrepos.get_mapping())))
 
     def test_fetch_invalid_svk_rev(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         uuid = SvnRaTransport(repos_url).get_uuid()
 
@@ -274,12 +279,14 @@ class TestFetchWorks(FetchTestCase):
         rf = FetchRevisionFinder(oldrepos, newrepos)
         rm = oldrepos._revmeta_provider.get_revision("trunk", 3)
         mapping = oldrepos.get_mapping()
-        self.assertEquals((uuid, "trunk", 3), rm.get_foreign_revid())
-        self.assertEquals((mapping.revision_id_foreign_to_bzr((uuid, "branches/trunk", 1)), ), rm.get_rhs_parents(mapping))
+        self.assertEquals((uuid, "trunk", 3), rm.metarev.get_foreign_revid())
+        self.assertEquals(
+            (mapping.revision_id_foreign_to_bzr((uuid, "branches/trunk", 1)), ),
+            rm.get_rhs_parents(mapping, rm.get_lhs_parent_revmeta(mapping)))
         self.assertEquals([], list(rf.find_rhs_parents([(rm, mapping)])))
 
     def test_fetch_file_branch(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_file("trunk")
         dc.close()
@@ -291,7 +298,7 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
 
     def test_fetch_special_char_edit(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -310,7 +317,7 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
 
     def test_fetch_special_char_child(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -324,7 +331,7 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
 
     def test_fetch_special_char_modify(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -343,7 +350,7 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
 
     def test_fetch_delete(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         foo = dc.add_dir("foo")
@@ -365,7 +372,7 @@ class TestFetchWorks(FetchTestCase):
         self.assertTrue(oldrepos.has_revision(oldrepos.generate_revision_id(2, "", mapping)))
 
     def test_fetch_delete_recursive(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         foo = dc.add_dir("foo")
@@ -383,12 +390,12 @@ class TestFetchWorks(FetchTestCase):
         self.copy_content(oldrepos, newrepos)
         mapping = oldrepos.get_mapping()
         tree = newrepos.revision_tree(oldrepos.generate_revision_id(1, "", mapping))
-        self.assertEquals(3, len(tree.inventory))
+        self.assertEquals(3, len(tree.all_file_ids()))
         tree = newrepos.revision_tree(oldrepos.generate_revision_id(2, "", mapping))
-        self.assertEquals(1, len(tree.inventory))
+        self.assertEquals(1, len(tree.all_file_ids()))
 
     def test_fetch_local(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         foo = dc.add_dir("foo")
@@ -419,12 +426,41 @@ class TestFetchWorks(FetchTestCase):
                     oldrepos.generate_revision_id(2, "", mapping))
             self.assertTrue(tree.has_filename("foo/bla"))
             self.assertTrue(tree.has_filename("foo"))
-            self.assertEqual("data", tree.get_file_by_path("foo/bla").read())
+            self.assertEqual("data", tree.get_file_text(tree.path2id("foo/bla"), "foo/bla"))
         finally:
             newrepos.unlock()
 
+    def test_fetch_mapping_upgrade(self):
+        from bzrlib.plugins.svn.mapping3.base import BzrSvnMappingv3
+        from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
+        repos_url = self.make_svn_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        br1 = dc.add_dir("trunk")
+        br2 = br1.add_dir("trunk/bloe")
+        br2.add_file("trunk/bloe/README").modify("BLA")
+        dc.close()
+
+        old_mapping = BzrSvnMappingv3(TrunkBranchingScheme(0))
+        self.client_set_revprop(repos_url, 1, SVN_REVPROP_BZR_MAPPING_VERSION, old_mapping.name)
+        self.client_set_revprop(repos_url, 1, SVN_REVPROP_BZR_ROOT, "trunk")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.open_dir("trunk").add_file("trunk/a").modify("LA")
+        dc.close()
+
+        oldrepos = Repository.open(repos_url)
+        to_bzrdir = BzrDir.create("f")
+        repo = to_bzrdir.create_repository()
+        revfinder = FetchRevisionFinder(oldrepos, repo)
+        new_mapping = oldrepos.get_mapping()
+        revfinder.find_all(new_mapping, TrunkLayout())
+        self.assertEquals(
+            [("trunk", 1, old_mapping), ("trunk", 2, new_mapping)],
+            [(rm.metarev.branch_path, rm.metarev.revnum, mapping) for (rm, mapping) in revfinder.needed])
+
     def test_fetch_move_root(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         br1 = dc.add_dir("branch1")
@@ -450,13 +486,13 @@ class TestFetchWorks(FetchTestCase):
         last_rev = oldrepos.generate_revision_id(3, "branch2",
             oldrepos.get_mapping())
         self.fetch(oldrepos, repo, last_rev)
-        self.assertEquals(repo.get_inventory(last_rev).root.file_id,
+        self.assertEquals(repo.revision_tree(last_rev).get_root_id(),
             oldrepos.get_mapping().generate_file_id(
                 (oldrepos.uuid, "branch1", 1), u""))
 
 
     def test_fetch_replace(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("bla").modify("data")
@@ -477,14 +513,14 @@ class TestFetchWorks(FetchTestCase):
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(2, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertNotEqual(inv1.path2id("bla"), inv2.path2id("bla"))
+        self.assertNotEqual(tree1.path2id("bla"), tree2.path2id("bla"))
 
     def test_fetch_copy_subdir(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -597,9 +633,9 @@ Node-action: delete
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertTrue(inv1.has_filename(u"x\xe1"))
+        self.assertTrue(tree1.has_filename(u"x\xe1"))
 
     def test_fetch_replace_with_subreplace(self):
         filename = os.path.join(self.test_dir, "dumpfile")
@@ -741,9 +777,9 @@ Node-copyfrom-path: u
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(3, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(3, "", mapping))
 
     def test_fetch_replace_self(self):
@@ -888,11 +924,11 @@ Node-copyfrom-path: bla
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(3, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(3, "", mapping))
-        self.assertEqual(inv1.path2id("bla"), inv2.path2id("bla"))
+        self.assertEqual(tree1.path2id("bla"), tree2.path2id("bla"))
 
     def test_fetch_replace_backup(self):
         filename = os.path.join(self.test_dir, "dumpfile")
@@ -1069,11 +1105,11 @@ Node-copyfrom-path: bla
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(3, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(3, "", mapping))
-        self.assertEqual(inv1.path2id("bla"), inv2.path2id("bla"))
+        self.assertEqual(tree1.path2id("bla"), tree2.path2id("bla"))
 
     def test_fetch_replace_unrelated(self):
         filename = os.path.join(self.test_dir, "dumpfile")
@@ -1211,11 +1247,11 @@ Node-copyfrom-path: x
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(4, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(4, "", mapping))
-        self.assertNotEqual(inv1.path2id("x"), inv2.path2id("x"))
+        self.assertNotEqual(tree1.path2id("x"), tree2.path2id("x"))
 
     def test_fetch_replace_related(self):
         filename = os.path.join(self.test_dir, "dumpfile")
@@ -1375,14 +1411,14 @@ Node-copyfrom-path: x
             oldrepos.generate_revision_id(1, "", mapping)))
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(5, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(5, "", mapping))
-        self.assertNotEqual(inv1.path2id("y"), inv2.path2id("y"))
+        self.assertNotEqual(tree1.path2id("y"), tree2.path2id("y"))
 
     def test_fetch_dir_replace_self(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         dc.add_dir("branches")
@@ -1427,7 +1463,7 @@ Node-copyfrom-path: x
         self.assertTrue(tree.path2id("dir1/file.txt") is not None)
 
     def test_fetch_dir_upgrade(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -1448,15 +1484,11 @@ Node-copyfrom-path: x
 
         branch = Branch.open("%s/branches/mybranch" % repos_url)
         mapping = oldrepos.get_mapping()
-        if mapping.is_branch("trunk/lib"):
-            self.assertEqual([oldrepos.generate_revision_id(1, "trunk/lib", mapping), oldrepos.generate_revision_id(2, "branches/mybranch", mapping)],
-                             branch.revision_history())
-        else:
-            self.assertEqual([oldrepos.generate_revision_id(2, "branches/mybranch", mapping)],
-                             branch.revision_history())
+        self.assertEqual(oldrepos.generate_revision_id(2, "branches/mybranch", mapping),
+                         branch.last_revision())
 
     def test_fetch_dir_from_non_branch(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         old_trunk = dc.add_dir("old-trunk")
@@ -1476,11 +1508,11 @@ Node-copyfrom-path: x
         self.copy_content(oldrepos, newrepos)
 
         branch = Branch.open("%s/trunk" % repos_url)
-        self.assertEqual([oldrepos.generate_revision_id(2, "trunk", oldrepos.get_mapping())],
-                         branch.revision_history())
+        self.assertEqual(oldrepos.generate_revision_id(2, "trunk", oldrepos.get_mapping()),
+                         branch.last_revision())
 
     def test_fetch_from_non_branch(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         ot = dc.add_dir("old-trunk")
@@ -1499,20 +1531,12 @@ Node-copyfrom-path: x
         self.copy_content(oldrepos, newrepos)
 
         branch = Branch.open("%s/trunk" % repos_url)
-        if oldrepos.get_mapping().is_branch("old-trunk"):
-            self.assertEqual([
-                oldrepos.generate_revision_id(1, "old-trunk", oldrepos.get_mapping()),
-                oldrepos.generate_revision_id(2, "trunk", oldrepos.get_mapping())],
-                         branch.revision_history())
-        else:
-            self.assertEqual([
-                oldrepos.generate_revision_id(2, "trunk", oldrepos.get_mapping())],
-                         branch.revision_history())
-
-
+        self.assertEqual(
+            oldrepos.generate_revision_id(2, "trunk", oldrepos.get_mapping()),
+            branch.last_revision())
 
     def test_fetch_branch_downgrade(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -1532,7 +1556,7 @@ Node-copyfrom-path: x
         self.copy_content(oldrepos, newrepos)
 
     def test_fetch_all(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -1577,7 +1601,7 @@ Node-copyfrom-path: x
             oldrepos.generate_revision_id(2, "", mapping)))
 
     def test_fetch_copy_root_id_kept(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir('trunk')
@@ -1594,7 +1618,7 @@ Node-copyfrom-path: x
         dc.close()
 
         repos = remote.SvnRemoteAccess(SvnRaTransport(repos_url),
-                format.SvnRemoteFormat()).find_repository()
+                remote.SvnRemoteFormat()).find_repository()
         repos.set_layout(TrunkLayout(0))
 
         mapping = repos.get_mapping()
@@ -1604,10 +1628,10 @@ Node-copyfrom-path: x
 
         self.assertEqual(
             mapping.generate_file_id((repos.uuid, "trunk", 1), u""),
-            tree.inventory.root.file_id)
+            tree.get_root_id())
 
     def test_fetch_odd(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -1638,7 +1662,7 @@ Node-copyfrom-path: x
         foobranch.open_file("branches/foobranch/hosts").modify('foohosts')
         dc.close()
 
-        repos = remote.SvnRemoteAccess(SvnRaTransport(repos_url), format.SvnRemoteFormat()).find_repository()
+        repos = remote.SvnRemoteAccess(SvnRaTransport(repos_url), remote.SvnRemoteFormat()).find_repository()
         repos.set_layout(TrunkLayout(0))
 
         mapping = repos.get_mapping()
@@ -1647,7 +1671,7 @@ Node-copyfrom-path: x
              repos.generate_revision_id(6, "branches/foobranch", mapping))
 
     def test_fetch_consistent(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         f = dc.add_file("bla")
@@ -1664,14 +1688,14 @@ Node-copyfrom-path: x
         self.copy_content(oldrepos, newrepos1)
         self.copy_content(oldrepos, newrepos2)
         mapping = oldrepos.get_mapping()
-        inv1 = newrepos1.get_inventory(
+        tree1 = newrepos1.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos2.get_inventory(
+        tree2 = newrepos2.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertEqual(inv1, inv2)
+        self.assertFalse(tree1.changes_from(tree2).has_changed())
 
     def test_fetch_executable(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         bla = dc.add_file("bla")
@@ -1690,13 +1714,13 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertTrue(inv1[inv1.path2id("bla")].executable)
-        self.assertTrue(inv1[inv1.path2id("blie")].executable)
+        self.assertTrue(tree1.is_executable(tree1.path2id("bla")))
+        self.assertTrue(tree1.is_executable(tree1.path2id("blie")))
 
     def test_fetch_executable_persists(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         bla = dc.add_file("bla")
@@ -1714,17 +1738,16 @@ Node-copyfrom-path: x
         newrepos = dir.create_repository()
         self.copy_content(oldrepos, newrepos)
         mapping = oldrepos.get_mapping()
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertTrue(inv1[inv1.path2id("bla")].executable)
-        inv2 = newrepos.get_inventory(
+        self.assertTrue(tree1.is_executable(tree1.path2id("bla")))
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv2[inv2.path2id("bla")].executable)
+        self.assertTrue(tree2.is_executable(tree2.path2id("bla")))
 
     def test_fetch_symlink(self):
-        if not has_symlinks():
-            return
-        repos_url = self.make_repository('d')
+        self.requireFeature(SymlinkFeature)
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("bla").modify("data")
@@ -1741,13 +1764,13 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        ie = inv1[inv1.path2id("mylink")]
-        self.assertEqual('symlink', ie.kind)
-        self.assertEqual('bla', ie.symlink_target)
-        self.assertEquals(None, ie.text_sha1)
-        key = (ie.file_id, ie.revision)
+        file_id = tree1.path2id("mylink")
+        self.assertEqual('symlink', tree1.kind(file_id))
+        self.assertEqual('bla', tree1.get_symlink_target(file_id))
+        self.assertEquals(None, tree1.get_file_sha1(file_id))
+        key = (file_id, tree1.get_file_revision(file_id))
         newrepos.lock_read()
         try:
             self.assertEquals(sha_string(""),
@@ -1756,10 +1779,8 @@ Node-copyfrom-path: x
             newrepos.unlock()
 
     def test_fetch_symlink_with_newlines(self):
-        if not has_symlinks():
-            return
-        raise KnownFailure("Bazaar doesn't support newlines in symlink targets (#219832)")
-        repos_url = self.make_repository('d')
+        self.requireFeature(SymlinkFeature)
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("bla").modify("data")
@@ -1771,17 +1792,22 @@ Node-copyfrom-path: x
         oldrepos = Repository.open(repos_url)
         dir = BzrDir.create("f")
         newrepos = dir.create_repository()
-        self.copy_content(oldrepos, newrepos)
+        try:
+            self.copy_content(oldrepos, newrepos)
+        except SymlinkTargetContainsNewline:
+            raise KnownFailure(
+                "Bazaar doesn't support newlines in symlink targets (#219832)")
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertEqual('symlink', inv1[inv1.path2id("mylink")].kind)
-        self.assertEqual('bla\nbar\nbla', inv1[inv1.path2id("mylink")].symlink_target)
+        self.assertEqual('symlink', tree1.kind(tree1.path2id("mylink")))
+        self.assertEqual('bla\nbar\nbla',
+            tree1.get_symlink_target(tree1.path2id("mylink")))
 
     def test_fetch_special_non_symlink(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         l = dc.add_file("mylink")
@@ -1797,12 +1823,12 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertEqual('file', inv1[inv1.path2id("mylink")].kind)
+        self.assertEqual('file', tree1.kind(tree1.path2id("mylink")))
 
     def test_fetch_special_unbecomes_symlink(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         l = dc.add_file("mylink")
@@ -1823,12 +1849,26 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(2, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertEqual('file', inv1[inv1.path2id("mylink")].kind)
+        self.assertEqual('file', tree1.kind(tree1.path2id("mylink")))
+
+        # Now test the same thing again as two separate copy_content calls, to
+        # check it also works if the symlink's "text" is not in the fetch
+        # text_cache.
+        dir2 = BzrDir.create("f2")
+        newrepos2 = dir2.create_repository()
+        self.copy_content(oldrepos, newrepos2,
+              oldrepos.generate_revision_id(1, "", mapping))
+        self.copy_content(oldrepos, newrepos2)
+        self.assertTrue(newrepos2.has_revision(
+            oldrepos.generate_revision_id(2, "", mapping)))
+        tree2 = newrepos2.revision_tree(
+                oldrepos.generate_revision_id(2, "", mapping))
+        self.assertEqual('file', tree2.kind(tree2.path2id("mylink")))
 
     def test_fetch_special_becomes_symlink(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         l = dc.add_file("mylink")
@@ -1849,13 +1889,16 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(2, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        raise KnownFailure("not allowing svn:special invalid files to be restored to symlinks yet")
-        self.assertEqual('symlink', inv1[inv1.path2id("mylink")].kind)
+        self.expectFailure(
+            "not allowing svn:special invalid files to be restored to symlinks "
+            "yet",
+            self.assertEqual,
+            'symlink', tree1.kind(tree1.path2id("mylink")))
 
     def test_fetch_symlink_kind_change(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("bla").modify("data")
@@ -1873,16 +1916,16 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        inv2 = newrepos.get_inventory(
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertEqual('file', inv1[inv1.path2id("mylink")].kind)
-        self.assertEqual('symlink', inv2[inv2.path2id("mylink")].kind)
-        self.assertEqual('bla', inv2[inv2.path2id("mylink")].symlink_target)
+        self.assertEqual('file', tree1.kind(tree1.path2id("mylink")))
+        self.assertEqual('symlink', tree2.kind(tree2.path2id("mylink")))
+        self.assertEqual('bla', tree2.get_symlink_target(tree2.path2id("mylink")))
 
     def test_fetch_executable_separate(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         dc.add_file("bla").modify("data")
@@ -1900,17 +1943,17 @@ Node-copyfrom-path: x
         mapping = oldrepos.get_mapping()
         self.assertTrue(newrepos.has_revision(
             oldrepos.generate_revision_id(1, "", mapping)))
-        inv1 = newrepos.get_inventory(
+        tree1 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(1, "", mapping))
-        self.assertFalse(inv1[inv1.path2id("bla")].executable)
-        inv2 = newrepos.get_inventory(
+        self.assertFalse(tree1.is_executable(tree1.path2id("bla")))
+        tree2 = newrepos.revision_tree(
                 oldrepos.generate_revision_id(2, "", mapping))
-        self.assertTrue(inv2[inv2.path2id("bla")].executable)
+        self.assertTrue(tree2.is_executable(tree2.path2id("bla")))
         self.assertEqual(oldrepos.generate_revision_id(2, "", mapping),
-                         inv2[inv2.path2id("bla")].revision)
+                         tree2.get_file_revision(tree2.path2id("bla")))
 
     def test_fetch_hidden(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -1943,7 +1986,7 @@ Node-copyfrom-path: x
         self.assertFalse(newrepos.has_revision(mapping.revision_id_foreign_to_bzr((oldrepos.uuid, "branches/brancha", 2))))
 
     def test_fetch_svk_merge(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -1974,7 +2017,7 @@ Node-copyfrom-path: x
         self.assertTrue(oldrepos.generate_revision_id(2, "branches/foo", mapping) in rev.parent_ids)
 
     def build_intertwined(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
         trunk.change_prop("bzr:revision-id:v3-trunk0", "1 revid1\n")
@@ -2036,7 +2079,7 @@ Node-copyfrom-path: x
                 set(newrepos.all_revision_ids()))
 
     def test_fetch_property_change_only(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
         trunk.add_file("trunk/bla").modify("data")
@@ -2069,7 +2112,7 @@ Node-copyfrom-path: x
             ]), set(newrepos.all_revision_ids()))
 
     def test_fetch_property_change_only_trunk(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
         trunk.add_file("trunk/bla").modify("data")
@@ -2105,7 +2148,7 @@ Node-copyfrom-path: x
 
     def test_fetch_replace_self_open(self):
         # Not actually a replace, merely just an open. Bug #308353
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -2134,14 +2177,14 @@ Node-copyfrom-path: x
         self.assertChangedPathsEquals({
             'trunk/adir': (u'R', 'trunk/adir', 2, NODE_DIR),
             'trunk/adir/foofile': (u'A', None, -1, NODE_FILE)},
-              oldrepos._revmeta_provider.get_revision("trunk", 3).paths)
+              oldrepos._revmeta_provider.get_revision("trunk", 3).metarev.paths)
         oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f")
         newrepos = dir.create_repository()
         self.copy_content(oldrepos, newrepos)
 
     def test_fetch_replace_self_open_older(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -2171,14 +2214,14 @@ Node-copyfrom-path: x
         self.assertChangedPathsEquals({
             'trunk/adir': (u'R', 'trunk/adir', 1, NODE_DIR),
             'trunk/adir/foofile': (u'A', None, -1, NODE_FILE)},
-              oldrepos._revmeta_provider.get_revision("trunk", 3).paths)
+              oldrepos._revmeta_provider.get_revision("trunk", 3).metarev.paths)
         oldrepos.set_layout(TrunkLayout(0))
         dir = BzrDir.create("f")
         newrepos = dir.create_repository()
         self.copy_content(oldrepos, newrepos)
 
     def test_fetch_crosscopy(self):
-        repos_url = self.make_repository('d')
+        repos_url = self.make_svn_repository('d')
 
         dc = self.get_commit_editor(repos_url)
         trunk = dc.add_dir("trunk")
@@ -2218,13 +2261,43 @@ Node-copyfrom-path: x
         lastrev = oldrepos.generate_revision_id(4, "branches/abranch", mapping)
         self.copy_content(oldrepos, newrepos, lastrev)
 
-        inventory = newrepos.get_inventory(lastrev)
+        tree = newrepos.revision_tree(lastrev)
         self.assertEqual(prevrev,
-                         inventory[inventory.path2id("bdir/afile")].revision)
+                         tree.get_file_revision(tree.path2id("bdir/afile")))
 
-        inventory = newrepos.get_inventory(prevrev)
+        tree = newrepos.revision_tree(prevrev)
         self.assertEqual(copyrev,
-                         inventory[inventory.path2id("bdir/stationary")].revision)
+                         tree.get_file_revision(tree.path2id("bdir/stationary")))
+
+    def test_fetch_different_parent_path(self):
+        repos_url = self.make_svn_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        myproj = dc.add_dir("myproj")
+        trunk = myproj.add_dir("myproj/trunk")
+        trunk.add_file("myproj/trunk/afile").modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("oldlayout")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        oldlayout = dc.open_dir("oldlayout")
+        oldlayout.add_dir("oldlayout/myproj", "myproj")
+        dc.delete("myproj")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_dir("trunk/myproj", "oldlayout/myproj/trunk")
+        oldlayout = dc.open_dir("oldlayout")
+        myproj = oldlayout.open_dir("oldlayout/myproj")
+        myproj.delete("oldlayout/myproj/trunk")
+        dc.close()
+
+        oldbranch = BzrDir.open(repos_url+"/trunk/myproj")
+        oldbranch.sprout("dc")
 
 
 class ChunksStartWithLinkTests(TestCase):
