@@ -16,11 +16,16 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Simple transport for accessing Subversion smart servers."""
 
-import itertools
 import stat
 import subvertpy
-from subvertpy import ERR_FS_NOT_DIRECTORY
+from subvertpy import (
+    AUTH_PARAM_DEFAULT_USERNAME,
+    AUTH_PARAM_DEFAULT_PASSWORD,
+    ERR_FS_NOT_DIRECTORY,
+    ERR_FS_NOT_FOUND,
+    )
 from subvertpy.client import get_config
+from subvertpy.ra import RemoteAccess
 import sys
 import urlparse
 urlparse.uses_netloc.extend(['svn+http', 'svn+https'])
@@ -34,7 +39,6 @@ from bzrlib import (
     )
 from bzrlib.errors import (
     BzrError,
-    DependencyNotPresent,
     FileExists,
     InvalidURL,
     NoSuchFile,
@@ -52,8 +56,6 @@ from bzrlib.transport import (
 
 import bzrlib.plugins.svn
 from bzrlib.plugins.svn.auth import (
-    AUTH_PARAM_DEFAULT_USERNAME,
-    AUTH_PARAM_DEFAULT_PASSWORD,
     create_auth_baton,
     )
 from bzrlib.plugins.svn.changes import (
@@ -66,12 +68,11 @@ from bzrlib.plugins.svn.errors import (
     convert_error,
     )
 
-ERR_MALFORMED_FILE = getattr(subvertpy, "ERR_MALFORMED_FILE", 200002)
 
 try:
     svn_config = get_config()
 except subvertpy.SubversionException, (msg, num):
-    if num == ERR_MALFORMED_FILE:
+    if num == subvertpy.ERR_MALFORMED_FILE:
         raise BzrError(msg)
     raise
 
@@ -87,47 +88,6 @@ def get_test_permutations():
     return []
 
 
-def dav_options(transport, url):
-    # FIXME: Integrate this into HttpTransport.options().
-    from bzrlib.transport.http._urllib import HttpTransport_urllib, Request
-    if isinstance(transport, HttpTransport_urllib):
-        req = Request('OPTIONS', url, accepted_errors=[200, 403, 404, 405])
-        req.follow_redirections = True
-        resp = transport._perform(req)
-        if resp.code == 404:
-            raise NoSuchFile(transport._path)
-        if resp.code in (403, 405):
-            raise subvertpy.SubversionException("version control not supported for remote URL", subvertpy.ERR_RA_DAV_NOT_VCC)
-        return resp.headers.getheaders('DAV')
-    else:
-        try:
-            from bzrlib.transport.http._pycurl import PyCurlTransport
-        except DependencyNotPresent:
-            pass
-        else:
-            import pycurl
-            from cStringIO import StringIO
-            if isinstance(transport, PyCurlTransport):
-                conn = transport._get_curl()
-                conn.setopt(pycurl.URL, url)
-                transport._set_curl_options(conn)
-                conn.setopt(pycurl.CUSTOMREQUEST, 'OPTIONS')
-                conn.setopt(pycurl.NOBODY, 1)
-                header = StringIO()
-                data = StringIO()
-                conn.setopt(pycurl.HEADERFUNCTION, header.write)
-                conn.setopt(pycurl.WRITEFUNCTION, data.write)
-                transport._curl_perform(conn, header)
-                code = conn.getinfo(pycurl.HTTP_CODE)
-                if code == 404:
-                    raise NoSuchFile(transport._path)
-                if code in (403, 405):
-                    raise subvertpy.SubversionException("version control not supported for remote URL", subvertpy.ERR_RA_DAV_NOT_VCC)
-                headers = transport._parse_headers(header)
-                return headers.getheaders('DAV')
-    raise NotImplementedError
-
-
 _warned_codeplex = False
 def warn_codeplex(host):
     global _warned_codeplex
@@ -147,26 +107,11 @@ def get_svn_ra_transport(bzr_transport):
     if ra_transport is not None:
         return ra_transport
 
-    svnbase = bzr_to_svn_url(bzr_transport.external_url())
-    # If this is a HTTP transport, use the existing connection to check
-    # that the remote end supports version control.
-    from bzrlib.transport.http import HttpTransportBase
-    if isinstance(bzr_transport, HttpTransportBase):
-        url = bzr_transport._unsplit_url(bzr_transport._unqualified_scheme,
-            None, None, bzr_transport._host, bzr_transport._port,
-            bzr_transport._path)
-        try:
-            dav_entries = dav_options(bzr_transport, url)
-        except NotImplementedError:
-            pass
-        else:
-            dav_entries = list(itertools.chain(*[entry.split(",") for entry in dav_entries]))
-            if not "version-control" in dav_entries:
-                raise subvertpy.SubversionException("version control not supported for remote URL", subvertpy.ERR_RA_DAV_NOT_VCC)
     # Save _svn_ra transport here so we don't have to connect again next time
     # we try to use bzr svn on this transport
     shared_connection = getattr(bzr_transport, "_shared_connection", None)
-    if shared_connection is not None and shared_connection.credentials is not None:
+    if (shared_connection is not None and
+        shared_connection.credentials is not None):
         if isinstance(shared_connection.credentials, dict):
             creds = shared_connection.credentials
         elif isinstance(shared_connection.credentials[0], dict):
@@ -175,7 +120,8 @@ def get_svn_ra_transport(bzr_transport):
             creds = None
     else:
         creds = None
-    ra_transport = SvnRaTransport(svnbase, credentials=creds)
+    ra_transport = SvnRaTransport(bzr_transport.external_url(),
+        credentials=creds)
     bzr_transport._svn_ra = ra_transport
     return ra_transport
 
@@ -212,8 +158,12 @@ def bzr_to_svn_url(url):
 
     This will possibly remove the svn+ prefix.
     """
+    url = urlutils.split_segment_parameters(url.strip("/"))[0]
     if url.startswith("readonly+"):
         url = url[len("readonly+"):]
+        readonly = True
+    else:
+        readonly = False
 
     if (url.startswith("svn+http://") or
         url.startswith("svn+https://")):
@@ -224,10 +174,10 @@ def bzr_to_svn_url(url):
     # The SVN libraries don't like trailing slashes...
     url = url.rstrip('/')
 
-    return url
+    return url, readonly
 
 
-def svn_to_bzr_url(url):
+def svn_to_bzr_url(url, readonly=False):
     """Convert a Subversion URL to a URL understood by Bazaar.
 
     This mainly involves fixing file:// URLs on Windows.
@@ -236,6 +186,8 @@ def svn_to_bzr_url(url):
         # Subversion URLs have only two // after file: on Windows
         if url.startswith("file://"):
             url = "file:///" + url[len("file://"):]
+    if readonly:
+        url = "readonly+" + url
     return url
 
 
@@ -254,26 +206,43 @@ class SubversionProgressReporter(object):
             assert progress < 100000
             self._last_progress = 0
         changed = progress - self._last_progress
-        assert changed >= 0, "changed was %d (%d -> %d)" % (changed, self._last_progress, progress)
+        if changed < 0:
+            raise AssertionError("changed was %d (%d -> %d)" % (changed, self._last_progress, progress))
         self._last_progress = progress
         ui.ui_factory.report_transport_activity(self, changed, None)
 
 
-def Connection(url, auth=None, config=None):
+def convert_relocate_error(url, num, msg):
+    """Convert a permanently moved error."""
+    # Try to guess the new url
+    if "'" in msg:
+        new_url = msg.split("'")[1]
+    elif "«" in msg:
+        new_url = msg[msg.index("»")+2:msg.index("«")]
+    else:
+        raise AssertionError("Unable to parse error message: %s" % msg)
+    raise RedirectRequested(source=_url_escape_uri(url),
+        target=_url_escape_uri(urlutils.join(url, new_url)),
+        is_permanent=True)
+
+
+def Connection(url, auth=None, config=None, readonly=False):
     # Subvertpy <= 0.6.3 has a bug in the reference counting of the
     # progress update function
     if subvertpy.__version__ >= (0, 6, 3):
         progress_cb = SubversionProgressReporter(url).update
     else:
         progress_cb = None
-    assert type(url) == str
+    assert type(url) == str, "URL not string: %r" % url
     try:
-        ret = subvertpy.ra.RemoteAccess(_url_escape_uri(url), auth=auth,
+        ret = RemoteAccess(_url_escape_uri(url), auth=auth,
                 client_string_func=bzrlib.plugins.svn.get_client_string,
                 progress_cb=progress_cb,
                 config=config)
         if 'transport' in debug.debug_flags:
             ret = MutteringRemoteAccess(ret)
+        if readonly:
+            ret = ReadonlyRemoteAccess(ret)
     except subvertpy.SubversionException, e:
         msg, num = e.args
         if num in (subvertpy.ERR_RA_SVN_REPOS_NOT_FOUND,):
@@ -285,16 +254,7 @@ def Connection(url, auth=None, config=None):
         if num == subvertpy.ERR_RA_ILLEGAL_URL:
             raise InvalidURL(url, msg)
         if num == subvertpy.ERR_RA_DAV_RELOCATED:
-            # Try to guess the new url
-            if "'" in msg:
-                new_url = msg.split("'")[1]
-            elif "«" in msg:
-                new_url = msg[msg.index("»")+2:msg.index("«")]
-            else:
-                raise AssertionError("Unable to parse error message: %s" % msg)
-            raise RedirectRequested(source=_url_escape_uri(url),
-                                    target=_url_escape_uri(urlutils.join(url, new_url)),
-                                    is_permanent=True)
+            raise convert_relocate_error(url, num, msg)
         raise convert_error(e)
     from bzrlib.plugins.svn import lazy_check_versions
     lazy_check_versions()
@@ -304,9 +264,10 @@ def Connection(url, auth=None, config=None):
 class ConnectionPool(object):
     """Collection of connections to a Subversion repository."""
 
-    def __init__(self, url):
+    def __init__(self, url, readonly=False):
         self.start_url = url
         self.connections = set()
+        self.readonly = readonly
         self.auth_baton = create_auth_baton(url)
 
     def set_credentials(self, credentials):
@@ -324,7 +285,8 @@ class ConnectionPool(object):
         try:
             c = self.connections.pop()
         except KeyError:
-            return Connection(self.start_url, self.auth_baton)
+            return Connection(self.start_url, self.auth_baton,
+                readonly=self.readonly)
         else:
             assert not c.busy, "busy connection in pool"
             return c
@@ -332,14 +294,14 @@ class ConnectionPool(object):
     def new(self, url):
         # Nothing available? Just pick an existing one and reparent:
         if len(self.connections) == 0:
-            return Connection(url, self.auth_baton)
+            return Connection(url, self.auth_baton, readonly=self.readonly)
         c = self.connections.pop()
         try:
             c.reparent(_url_escape_uri(url))
             return c
         except NotImplementedError:
             self.connections.add(c)
-            return Connection(url, self.auth_baton)
+            return Connection(url, self.auth_baton, readonly=self.readonly)
         except:
             self.connections.add(c)
             raise
@@ -379,20 +341,23 @@ class SvnRaTransport(Transport):
 
     @convert_svn_error
     def __init__(self, url, from_transport=None, credentials=None):
-        bzr_url = url
-        self.svn_url = bzr_to_svn_url(url)
-        Transport.__init__(self, bzr_url)
-        host = urlutils.parse_url(bzr_url)[3]
+        self.svn_url, readonly = bzr_to_svn_url(url)
+        Transport.__init__(self, url)
+        host = urlutils.parse_url(url)[3]
         if host.endswith(".codeplex.com"):
             warn_codeplex(host)
         if from_transport is None:
-            self.connections = ConnectionPool(bzr_url)
+            self.connections = ConnectionPool(self.svn_url,
+                readonly=readonly)
             if credentials is not None:
                 assert isinstance(credentials, dict)
                 self.connections.set_credentials(credentials)
             # Make sure that the URL is valid by connecting to it.
             self.connections.add(self.connections.get(self.svn_url))
         else:
+            if readonly != from_transport.is_readonly():
+                raise AssertionError("readonly %r != %r (%r)" % (readonly,
+                        from_transport.is_readonly(), from_transport))
             self.connections = from_transport.connections
             if credentials is not None:
                 assert isinstance(credentials, dict)
@@ -403,6 +368,9 @@ class SvnRaTransport(Transport):
         from bzrlib.plugins.svn import lazy_check_versions
         lazy_check_versions()
 
+    def is_readonly(self):
+        return self.connections.readonly
+
     def get_any_connection(self):
         return self.connections.get_any()
 
@@ -456,11 +424,8 @@ class SvnRaTransport(Transport):
                 self.st_mtime = svn_stat['time']
         conn, relpath = self.get_path_connection(path)
         try:
-            stat_fn = getattr(conn, "stat", None)
-            if stat_fn is None:
-                raise TransportNotPossible("stat not supported in this version")
             try:
-                return StatResult(stat_fn(relpath, -1))
+                return StatResult(conn.stat(relpath, -1))
             except NotImplementedError:
                 raise TransportNotPossible("stat not supported in this version")
         finally:
@@ -512,63 +477,16 @@ class SvnRaTransport(Transport):
             conn, paths = self.get_paths_connection(paths)
         else:
             conn = self.get_connection()
-        try:
-            return conn.iter_log(paths, from_revnum, to_revnum, limit,
-                discover_changed_paths=discover_changed_paths,
-                strict_node_history=strict_node_history,
-                include_merged_revisions=include_merged_revisions,
-                revprops=revprops)
-        except AttributeError:
-            # Older subvertpy, fall back to iterator wrapper.
-            pass
 
-        from threading import Thread, Semaphore
+        return conn.iter_log(paths, from_revnum, to_revnum, limit,
+            discover_changed_paths=discover_changed_paths,
+            strict_node_history=strict_node_history,
+            include_merged_revisions=include_merged_revisions,
+            revprops=revprops)
 
-        class logfetcher(Thread):
-            def __init__(self, transport, conn, *args, **kwargs):
-                Thread.__init__(self)
-                self.setDaemon(True)
-                self.transport = transport
-                self.args = args
-                self.kwargs = kwargs
-                self.pending = []
-                self.conn = conn
-                self.semaphore = Semaphore(0)
-                self.busy = False
-
-            def next(self):
-                self.semaphore.acquire()
-                ret = self.pending.pop(0)
-                if isinstance(ret, Exception):
-                    raise ret
-                return ret
-
-            def run(self):
-                assert not self.busy, "already running"
-                self.busy = True
-                def rcvr(orig_paths, revision, revprops, has_children=None):
-                    self.pending.append((orig_paths, revision, revprops, has_children))
-                    self.semaphore.release()
-                try:
-                    try:
-                        self.conn.get_log(rcvr, *self.args, **self.kwargs)
-                        self.pending.append(None)
-                    except Exception, e:
-                        self.pending.append(e)
-                finally:
-                    self.pending.append(Exception("Some exception was not handled"))
-                    self.semaphore.release()
-                    self.transport.add_connection(self.conn)
-        fetcher = logfetcher(self, conn, paths, from_revnum, to_revnum,
-                limit, discover_changed_paths=discover_changed_paths,
-                strict_node_history=strict_node_history,
-                include_merged_revisions=include_merged_revisions,
-                revprops=revprops)
-        fetcher.start()
-        return iter(fetcher.next, None)
-
-    def get_log(self, rcvr, paths, from_revnum, to_revnum, limit, discover_changed_paths,
-                strict_node_history, include_merged_revisions, revprops):
+    def get_log(self, rcvr, paths, from_revnum, to_revnum, limit,
+            discover_changed_paths, strict_node_history,
+            include_merged_revisions, revprops):
 
         if paths is not None:
             conn, paths = self.get_paths_connection(paths)
@@ -584,7 +502,9 @@ class SvnRaTransport(Transport):
             except subvertpy.SubversionException, e:
                 msg, num = e.args
                 if num == subvertpy.ERR_RA_DAV_REQUEST_FAILED:
-                    return DavRequestFailed(msg)
+                    raise DavRequestFailed(msg)
+                elif num == subvertpy.ERR_RA_DAV_RELOCATED:
+                    raise convert_relocate_error(conn.url, num, msg)
                 else:
                     raise e
         finally:
@@ -612,10 +532,13 @@ class SvnRaTransport(Transport):
         finally:
             self.add_connection(conn)
 
-    def get_file_revs(self, path, start_revnum, end_revnum, handler):
+    def get_file_revs(self, path, start_revnum, end_revnum, handler,
+                      include_merged_revisions=False):
+        assert type(path) is str
         conn, relpath = self.get_path_connection(path)
         try:
-            return conn.get_file_revs(relpath, start_revnum, end_revnum, handler)
+            return conn.get_file_revs(relpath, start_revnum, end_revnum, handler,
+                                      include_merged_revisions)
         finally:
             self.add_connection(conn)
 
@@ -636,35 +559,38 @@ class SvnRaTransport(Transport):
         finally:
             self.add_connection(conn)
 
+    @property
+    def repos_path(self):
+        return self.svn_url[len(self.get_repos_root()):].strip("/")
+
     @convert_svn_error
     def create_prefix(self):
         create_branch_prefix(self.clone_root(),
                 {"svn:log": "Creating prefix"},
-                self.svn_url[len(self.get_repos_root()):].strip("/").split("/")[:-1])
+                self.repos_path.split("/")[:-1])
 
     @convert_svn_error
     def mkdir(self, relpath, message="Creating directory"):
-        conn = self.get_connection()
+        relpath = urlutils.join(self.repos_path, relpath)
+        dirname, basename = urlutils.split(relpath)
+        conn = self.get_connection(dirname.strip("/"))
         try:
             ce = conn.get_commit_editor({"svn:log": message})
             try:
                 node = ce.open_root(-1)
-                batons = relpath.split("/")
-                toclose = [node]
-                for i in range(len(batons)):
-                    node = node.open_directory("/".join(batons[:i]), -1)
-                    toclose.append(node)
-                toclose.append(node.add_directory(relpath, None, -1))
-                for c in reversed(toclose):
-                    c.close()
-                ce.close()
+                node.add_directory(relpath, None, -1).close()
+                node.close()
             except subvertpy.SubversionException, (msg, num):
                 ce.abort()
+                if num == ERR_FS_NOT_FOUND:
+                    raise NoSuchFile(msg)
                 if num == ERR_FS_NOT_DIRECTORY:
                     raise NoSuchFile(msg)
                 if num == subvertpy.ERR_FS_ALREADY_EXISTS:
                     raise FileExists(msg)
                 raise
+            else:
+                ce.close()
         finally:
             self.add_connection(conn)
 
@@ -721,7 +647,10 @@ class SvnRaTransport(Transport):
         return self.PhonyLock() # FIXME
 
     def clone_root(self):
-        return SvnRaTransport(self.get_repos_root(), self)
+        url = self.get_repos_root()
+        if self.is_readonly():
+            url = "readonly+" + url
+        return SvnRaTransport(url, self)
 
     def clone(self, offset=None):
         """See Transport.clone()."""
@@ -744,7 +673,83 @@ class SvnRaTransport(Transport):
         return urlutils.join(self.base, relpath)
 
 
+class ReadonlyRemoteAccess(object):
+
+    busy = property(lambda self: self.actual.busy)
+    url = property(lambda self: self.actual.url)
+
+    def __init__(self, actual):
+        self.actual = actual
+
+    def check_path(self, path, revnum):
+        return self.actual.check_path(path, revnum)
+
+    def stat(self, path, revnum):
+        return self.actual.stat(path, revnum)
+
+    def has_capability(self, cap):
+        return self.actual.has_capability(cap)
+
+    def get_uuid(self):
+        return self.actual.get_uuid()
+
+    def get_repos_root(self):
+        return self.actual.get_repos_root()
+
+    def get_latest_revnum(self):
+        return self.actual.get_latest_revnum()
+
+    def get_log(self, *args, **kwargs):
+        return self.actual.get_log(*args, **kwargs)
+
+    def iter_log(self, *args, **kwargs):
+        return self.actual.iter_log(*args, **kwargs)
+
+    def change_rev_prop(self, revnum, name, value):
+        raise TransportNotPossible('readonly transport')
+
+    def get_dir(self, *args, **kwargs):
+        return self.actual.get_dir(*args, **kwargs)
+
+    def get_file(self, *args, **kwargs):
+        return self.actual.get_file(*args, **kwargs)
+
+    def get_file_revs(self, *args, **kwargs):
+        return self.actual.get_file_revs(*args, **kwargs)
+
+    def revprop_list(self, revnum):
+        return self.actual.revprop_list(revnum)
+
+    def get_locations(self, path, peg_revnum, revnums):
+        return self.actual.get_locations(path, peg_revnum, revnums)
+
+    def do_update(self, revnum, path, start_empty, editor):
+        return self.actual.do_update(revnum, path, start_empty, editor)
+
+    def do_diff(self, *args, **kwargs):
+        return self.actual.do_diff(*args, **kwargs)
+
+    def do_switch(self, *args, **kwargs):
+        return self.actual.do_switch(*args, **kwargs)
+
+    def reparent(self, url):
+        return self.actual.reparent(url)
+
+    def get_commit_editor(self, *args, **kwargs):
+        raise TransportNotPossible('readonly transport')
+
+    def rev_proplist(self, revnum):
+        return self.actual.rev_proplist(revnum)
+
+    def replay_range(self, *args, **kwargs):
+        return self.actual.replay_range(*args, **kwargs)
+
+    def replay(self, *args, **kwargs):
+        return self.actual.replay(*args, **kwargs)
+
+
 class MutteringRemoteAccess(object):
+    """Trivial RemoteAccess wrapper that mutters all activity."""
 
     busy = property(lambda self: self.actual.busy)
     url = property(lambda self: self.actual.url)
@@ -758,10 +763,7 @@ class MutteringRemoteAccess(object):
 
     def stat(self, path, revnum):
         mutter('svn stat -r%d %s' % (revnum, path))
-        stat = getattr(self.actual, "stat", None)
-        if stat is None:
-            raise NotImplementedError(self.stat)
-        return stat(path, revnum)
+        return self.actual.stat(path, revnum)
 
     def has_capability(self, cap):
         return self.actual.has_capability(cap)
