@@ -63,12 +63,16 @@ class SubversionTree(object):
         return True
 
     def lookup_id(self, path):
-        """Lookup the idmap data for a path.
+        """Lookup the file id and text revision for a path.
 
-        :param path: Path to look up.
-        :return: Tuple with file id and text revision
+        :param path: Unicode path
+        :raises KeyError: raised if path was not found
+        :return: tuple with file id and text revision
         """
-        raise NotImplementedError(self.lookup_id)
+        return self.id_map.lookup(self.mapping, path)[:2]
+
+    def lookup_path(self, file_id):
+        return self.id_map.reverse_lookup(self.mapping, file_id)
 
     def _get_rules_searcher(self, default_searcher):
         """Get the RulesSearcher for this tree given the default one."""
@@ -76,6 +80,92 @@ class SubversionTree(object):
             [SvnRulesSearcher(self), default_searcher])
 
 
+class SvnRevisionTreeCommon(SubversionTree,RevisionTree):
+
+    def has_or_had_id(self, file_id):
+        return self.has_id(file_id)
+
+    def id2path(self, file_id):
+        try:
+            return self.lookup_path(file_id)
+        except KeyError:
+            raise errors.NoSuchId(self, file_id)
+
+    def has_id(self, file_id):
+        try:
+            self.id2path(file_id)
+        except errors.NoSuchId:
+            return False
+        else:
+            return True
+
+    def path2id(self, path):
+        try:
+            file_id, revision_id = self.lookup_id(path)
+        except KeyError:
+            return None
+        else:
+            return file_id
+
+    def get_root_id(self):
+        return self.path2id("")
+
+    def _comparison_data(self, entry, path):
+        # FIXME
+        if entry is None:
+            return None, False, None
+        return entry.kind, entry.executable, None
+
+    def get_file_mtime(self, file_id, path=None):
+        """See Tree.get_file_mtime."""
+        if not path:
+            path = self.id2path(file_id)
+        revid = self.get_file_revision(file_id, path)
+        try:
+            rev = self._repository.get_revision(revid)
+        except errors.NoSuchRevision:
+            raise errors.FileTimestampUnavailable(path)
+        return rev.timestamp
+
+    def is_executable(self, file_id, path=None):
+        if path is None:
+            path = self.id2path(file_id)
+        props = self.get_file_properties(file_id, path)
+        if props.has_key(properties.PROP_SPECIAL):
+            text = self.get_file_stream_by_path(path)
+            if text.read(5) == "link ":
+                return False
+        return props.has_key(properties.PROP_EXECUTABLE)
+
+    def get_symlink_target(self, file_id, path=None):
+        if path is None:
+            path = self.id2path(file_id)
+        props = self.get_file_properties(file_id, path)
+        if not props.has_key(properties.PROP_SPECIAL):
+            return None
+        text = self.get_file_stream_by_path(path).read()
+        if not text.startswith("link "):
+            return None
+        return text[len("link "):].decode("utf-8")
+
+    def get_file_sha1(self, file_id, path=None, stat_value=None):
+        return osutils.sha_string(self.get_file_text(file_id, path))
+
+    def get_file_revision(self, file_id, path=None):
+        if path is None:
+            path = self.id2path(file_id)
+        file_id, file_revision = self.lookup_id(path)
+        return file_revision
+
+    def iter_files_bytes(self, file_ids):
+        for file_id, identifier in file_ids:
+            cur_file = (self.get_file_text(file_id),)
+            yield identifier, cur_file
+
+    def get_file_stream_by_path(self, path):
+        raise NotImplementedError(self.get_file_stream_by_path)
+
+
 # This maps SVN names for eol-styles to bzr names:
 eol_style = {
     "native": "native",
@@ -119,7 +209,7 @@ def inventory_add_external(inv, parent_id, path, revid, ref_revnum, url):
     :param parent_id: File id of directory the entry was set on.
     :param path: Path of the entry, relative to entry with parent_id.
     :param revid: Revision to store in newly created inventory entries.
-    :param ref_revnum: Referenced revision of tree that's being referenced, or
+    :param ref_revnum: Referenced *svn* revision of tree that's being referenced, or
         None if no specific revision is being referenced.
     :param url: URL of referenced tree.
     """
@@ -138,36 +228,45 @@ def inventory_add_external(inv, parent_id, path, revid, ref_revnum, url):
                                  parent_id=parent.file_id))
                 parent.revision = revid
 
+    # FIXME: This can only be a svn branch, so use that fact.
     reference_branch = Branch.open(url)
     file_id = reference_branch.get_root_id()
     ie = TreeReference(file_id, name, parent.file_id, revision=revid)
     if ref_revnum is not None:
-        ie.reference_revision = reference_branch.get_rev_id(ref_revnum)
+        ie.reference_revision = reference_branch.generate_revision_id(ref_revnum)
     else:
         ie.reference_revision = CURRENT_REVISION
     inv.add(ie)
 
 
-class SvnRevisionTree(SubversionTree,RevisionTree):
+class SvnRevisionTree(SvnRevisionTreeCommon):
     """A tree that existed in a historical Subversion revision."""
 
     def __init__(self, repository, revision_id):
         self._repository = repository
         self._revision_id = revision_id
         self._revmeta, self.mapping = repository._get_revmeta(revision_id)
-        self._inventory = Inventory()
-        self._inventory.revision_id = revision_id
+        self._bzr_inventory = None
         self._rules_searcher = None
         self.id_map = repository.get_fileid_map(self._revmeta, self.mapping)
-        editor = TreeBuildEditor(self)
-        self.file_data = {}
         self.file_properties = {}
-        root_repos = repository.transport.get_svn_repos_root()
-        conn = repository.transport.get_connection()
+        self.transport = repository.transport.clone(
+            self._revmeta.metarev.branch_path)
+
+    @property
+    def inventory(self):
+        # FIXME
+        if self._bzr_inventory is not None:
+            return self._bzr_inventory
+        self._bzr_inventory = Inventory()
+        self._bzr_inventory.revision_id = self.get_revision_id()
+        root_repos = self._repository.transport.get_svn_repos_root()
+        editor = TreeBuildEditor(self)
+        conn = self._repository.transport.get_connection()
         try:
             reporter = conn.do_switch(
-                self._revmeta.revnum, "", True,
-                urlutils.join(root_repos, self._revmeta.branch_path).rstrip("/"), editor)
+                self._revmeta.metarev.revnum, "", True,
+                urlutils.join(root_repos, self._revmeta.metarev.branch_path).rstrip("/"), editor)
             try:
                 reporter.set_path("", 0, True, None)
                 reporter.finish()
@@ -175,42 +274,105 @@ class SvnRevisionTree(SubversionTree,RevisionTree):
                 reporter.abort()
                 raise
         finally:
-            repository.transport.add_connection(conn)
+            self._repository.transport.add_connection(conn)
+        return self._bzr_inventory
 
-    def lookup_id(self, path):
-        return self.id_map.lookup(self.mapping, path)[:2]
+    def all_file_ids(self):
+        # FIXME
+        return set(self.inventory)
+
+    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
+        # FIXME
+        return self.inventory.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids, yield_parents=yield_parents)
+
+    def kind(self, file_id, path=None):
+        if path is None:
+            path = self.id2path(file_id)
+        stream = StringIO()
+        try:
+            (fetched_rev, props) = self.transport.get_file(path.encode("utf-8"),
+                    stream, self._revmeta.metarev.revnum)
+        except subvertpy.SubversionException, (_, num):
+            if num == subvertpy.ERR_FS_NOT_FILE:
+                return "directory"
+            raise
+        stream.seek(0)
+        if props.has_key(properties.PROP_SPECIAL) and stream.read(5) == "link ":
+            return "symlink"
+        return "file"
+
+    def get_file_size(self, file_id, path=None):
+        if path is None:
+            path = self.id2path(file_id)
+        # FIXME: More efficient implementation?
+        return self.inventory[file_id].text_size
+
+    def list_files(self, include_root=False, from_dir=None, recursive=True):
+        # FIXME
+        # The only files returned by this are those from the version
+        inv = self.inventory
+        if from_dir is None:
+            from_dir_id = None
+        else:
+            from_dir_id = inv.path2id(from_dir)
+            if from_dir_id is None:
+                # Directory not versioned
+                return
+        entries = inv.iter_entries(from_dir=from_dir_id, recursive=recursive)
+        if inv.root is not None and not include_root and from_dir is None:
+            # skip the root for compatability with the current apis.
+            entries.next()
+        for path, entry in entries:
+            yield path, 'V', entry.kind, entry.file_id, entry
+
+    def get_file(self, file_id, path=None):
+        if path is None:
+            path = self.id2path(file_id)
+        stream = StringIO()
+        (fetched_rev, props) = self.transport.get_file(path.encode("utf-8"),
+                stream, self._revmeta.metarev.revnum)
+        if props.has_key(properties.PROP_SPECIAL) and stream.read(5) == "link ":
+            return StringIO()
+        stream.seek(0)
+        return stream
+
+    def get_file_stream_by_path(self, path):
+        stream = StringIO()
+        (fetched_rev, props) = self.transport.get_file(path.encode("utf-8"),
+                stream, self._revmeta.metarev.revnum)
+        stream.seek(0)
+        return stream
 
     def get_file_text(self, file_id, path=None):
-        return self.file_data[file_id]
-
-    def iter_files_bytes(self, desired_files):
-        """Iterate through file contents.
-
-        Files will not necessarily be returned in the order they occur in
-        desired_files.  No specific order is guaranteed.
-
-        Yields pairs of identifier, bytes_iterator.  identifier is an opaque
-        value supplied by the caller as part of desired_files.  It should
-        uniquely identify the file version in the caller's context.  (Examples:
-        an index number or a TreeTransform trans_id.)
-
-        bytes_iterator is an iterable of bytestrings for the file.  The
-        kind of iterable and length of the bytestrings are unspecified, but for
-        this implementation, it is a tuple containing a single bytestring with
-        the complete text of the file.
-
-        :param desired_files: a list of (file_id, identifier) pairs
-        """
-        for file_id, identifier in desired_files:
-            # We wrap the string in a tuple so that we can return an iterable
-            # of bytestrings.  (Technically, a bytestring is also an iterable
-            # of bytestrings, but iterating through each character is not
-            # performant.)
-            cur_file = (self.get_file_text(file_id),)
-            yield identifier, cur_file
+        my_file = self.get_file(file_id, path)
+        try:
+            return my_file.read()
+        finally:
+            my_file.close()
 
     def get_file_properties(self, file_id, path=None):
-        return self.file_properties[file_id]
+        if path is None:
+            path = self.id2path(file_id)
+        encoded_path = path.encode("utf-8")
+        try:
+            (fetched_rev, props) = self.transport.get_file(encoded_path,
+                    StringIO(), self._revmeta.metarev.revnum)
+        except subvertpy.SubversionException, (_, num):
+            if num == subvertpy.ERR_FS_NOT_FILE:
+                (dirents, fetched_rev, props) = self.transport.get_dir(
+                    encoded_path, self._revmeta.metarev.revnum)
+            else:
+                raise
+        return props
+
+    def has_filename(self, path):
+        kind = self.transport.check_path(path.encode("utf-8"), self._revmeta.metarev.revnum)
+        return kind in (subvertpy.NODE_DIR, subvertpy.NODE_FILE)
+
+    def __repr__(self):
+        return "<%s for %r, revision_id=%s>" % (self.__class__.__name__,
+            self._revmeta, self.get_revision_id())
 
 
 class TreeBuildEditor(object):
@@ -226,7 +388,7 @@ class TreeBuildEditor(object):
 
     def open_root(self, revnum):
         file_id, revision_id = self.tree.lookup_id("")
-        ie = self.tree._inventory.add_path("", 'directory', file_id)
+        ie = self.tree._bzr_inventory.add_path("", 'directory', file_id)
         ie.revision = revision_id
         return DirectoryTreeEditor(self.tree, file_id)
 
@@ -247,7 +409,7 @@ class DirectoryTreeEditor(object):
     def add_directory(self, path, copyfrom_path=None, copyfrom_revnum=-1):
         path = path.decode("utf-8")
         file_id, revision_id = self.tree.lookup_id(path)
-        ie = self.tree._inventory.add_path(path, 'directory', file_id)
+        ie = self.tree._bzr_inventory.add_path(path, 'directory', file_id)
         ie.revision = revision_id
         return DirectoryTreeEditor(self.tree, file_id)
 
@@ -308,7 +470,6 @@ class FileTreeEditor(object):
             mutter('unsupported file property %r', name)
 
     def close(self, checksum=None):
-
         if self.file_stream:
             self.file_stream.seek(0)
             file_data = self.file_stream.read()
@@ -319,9 +480,11 @@ class FileTreeEditor(object):
             self.is_symlink = (self.is_special and file_data.startswith("link "))
 
         if self.is_symlink:
-            ie = self.tree._inventory.add_path(self.path, 'symlink', self.file_id)
+            ie = self.tree._bzr_inventory.add_path(self.path, 'symlink',
+                self.file_id)
         else:
-            ie = self.tree._inventory.add_path(self.path, 'file', self.file_id)
+            ie = self.tree._bzr_inventory.add_path(self.path, 'file',
+                self.file_id)
         ie.revision = self.revision_id
 
         actual_checksum = md5(file_data).hexdigest()
@@ -333,7 +496,6 @@ class FileTreeEditor(object):
         else:
             ie.text_sha1 = osutils.sha_string(file_data)
             ie.text_size = len(file_data)
-            self.tree.file_data[self.file_id] = file_data
             ie.executable = self.is_executable
 
         self.file_stream = None
@@ -343,33 +505,53 @@ class FileTreeEditor(object):
         return delta.apply_txdelta_handler("", self.file_stream)
 
 
-class SvnBasisTree(SubversionTree,RevisionTree):
+class SvnBasisTree(SvnRevisionTreeCommon):
     """Optimized version of SvnRevisionTree."""
 
     def __repr__(self):
         return "<%s for '%r'>" % (self.__class__.__name__, self.workingtree)
 
     def __init__(self, workingtree):
-        mutter("opening basistree for %r at %d",
-                workingtree, workingtree.base_revnum)
+        mutter("opening basistree for %r", workingtree)
         self.workingtree = workingtree
-        self._inventory = Inventory(root_id=None)
+        self._bzr_inventory = None
         self._repository = workingtree.branch.repository
+        self.id_map = self.workingtree.basis_idmap
+        self.mapping = self.workingtree.branch.mapping
+        self._real_tree = None
 
+    def get_file_verifier(self, file_id, path=None, stat_value=None):
+        if path is None:
+            path = self.id2path(file_id)
+        root_adm = self.workingtree._get_wc(write_lock=False)
+        try:
+            entry = self.workingtree._get_entry(root_adm, path)
+            return ("MD5", entry.checksum)
+        finally:
+            root_adm.close()
+
+    @property
+    def inventory(self):
+        if self._bzr_inventory is not None:
+            return self._bzr_inventory
+        self._bzr_inventory = Inventory(root_id=None)
+        if self.get_root_id() is None:
+            return self._bzr_inventory
         def add_file_to_inv(relpath, id, revid, adm):
             assert isinstance(relpath, unicode)
-            (propchanges, props) = adm.get_prop_diffs(self.workingtree.abspath(relpath).encode("utf-8"))
+            (propchanges, props) = adm.get_prop_diffs(
+                self.workingtree.abspath(relpath).encode("utf-8"))
             if props.has_key(properties.PROP_SPECIAL):
-                is_symlink = (self.get_file_byname(relpath).read(5) == "link ")
+                is_symlink = (self.get_file_stream_by_path(relpath).read(5) == "link ")
             else:
                 is_symlink = False
 
             if is_symlink:
-                ie = self._inventory.add_path(relpath, 'symlink', id)
-                ie.symlink_target = self.get_file_byname(relpath).read()[len("link "):]
+                ie = self._bzr_inventory.add_path(relpath, 'symlink', id)
+                ie.symlink_target = self.get_file_stream_by_path(relpath).read()[len("link "):]
             else:
-                ie = self._inventory.add_path(relpath, 'file', id)
-                data = osutils.fingerprint_file(self.get_file_byname(relpath))
+                ie = self._bzr_inventory.add_path(relpath, 'file', id)
+                data = osutils.fingerprint_file(self.get_file_stream_by_path(relpath))
                 ie.text_sha1 = data['sha1']
                 ie.text_size = data['size']
                 ie.executable = props.has_key(properties.PROP_EXECUTABLE)
@@ -383,7 +565,8 @@ class SvnBasisTree(SubversionTree,RevisionTree):
             if entry.schedule in (wc.SCHEDULE_NORMAL,
                                   wc.SCHEDULE_DELETE,
                                   wc.SCHEDULE_REPLACE):
-                return self.lookup_id(workingtree.unprefix(relpath.decode("utf-8")))
+                return self.lookup_id(
+                    self.workingtree.unprefix(relpath.decode("utf-8")))
             return (None, None)
 
         def add_dir_to_inv(relpath, adm, parent_id):
@@ -391,14 +574,14 @@ class SvnBasisTree(SubversionTree,RevisionTree):
             entries = adm.entries_read(False)
             entry = entries[""]
             (id, revid) = find_ids(entry)
-            if id == None:
+            if id is None:
                 return
 
             # First handle directory itself
-            ie = self._inventory.add_path(relpath, 'directory', id)
+            ie = self._bzr_inventory.add_path(relpath, 'directory', id)
             ie.revision = revid
             if relpath == u"":
-                self._inventory.revision_id = revid
+                self._bzr_inventory.revision_id = revid
 
             for name, entry in entries.iteritems():
                 if name == "":
@@ -427,33 +610,40 @@ class SvnBasisTree(SubversionTree,RevisionTree):
                     if subid is not None:
                         add_file_to_inv(subrelpath, subid, subrevid, adm)
 
-        adm = workingtree._get_wc()
+        adm = self.workingtree._get_wc()
         try:
             add_dir_to_inv(u"", adm, None)
         finally:
             adm.close()
+        return self._bzr_inventory
 
-    def lookup_id(self, path):
-        return self.workingtree.basis_idmap.lookup(self.workingtree.branch.mapping, path)[:2]
+    def has_filename(self, path):
+        try:
+            self.lookup_id(path)
+        except KeyError:
+            return False
+        else:
+            return True
 
     def get_revision_id(self):
         """See Tree.get_revision_id()."""
         return self.workingtree.last_revision()
 
-    def abspath(self, relpath):
-        assert isinstance(relpath, unicode)
-        return wc.get_pristine_copy_path(self.workingtree.abspath(relpath).encode("utf-8")).decode("utf-8")
-
-    def get_file_byname(self, name):
-        """See Tree.get_file_byname()."""
+    def get_file_stream_by_path(self, name):
+        """See Tree.get_file_stream_by_path()."""
         assert isinstance(name, unicode)
-        return open(self.abspath(name).encode(osutils._fs_enc))
+        wt_path = self.workingtree.abspath(name).encode("utf-8")
+        return wc.get_pristine_contents(wt_path)
+
+    def kind(self, file_id, path=None):
+        # FIXME
+        return self.inventory[file_id].kind
 
     def get_file_text(self, file_id, path=None):
         """See Tree.get_file_text()."""
         if path is None:
             path = self.id2path(file_id)
-        return self.get_file_byname(path).read()
+        return self.get_file_stream_by_path(path).read()
 
     def get_file_properties(self, file_id, path=None):
         """See SubversionTree.get_file_properties()."""
@@ -473,9 +663,29 @@ class SvnBasisTree(SubversionTree,RevisionTree):
         return orig_props
 
     def annotate_iter(self, file_id, default_revision=CURRENT_REVISION):
-        return self.workingtree.branch.repository._annotate(urlutils.join(self.workingtree.branch.get_branch_path(), self.inventory.id2path(file_id)).strip("/"), self.workingtree.base_revnum,  file_id, self.get_revision_id(), self.workingtree.branch.mapping)
+        from bzrlib.plugins.svn.annotate import Annotater
+        annotater = Annotater(self.workingtree.branch.repository, file_id)
+        annotater.check_file_revs(
+            self.get_revision_id(), self.workingtree.get_branch_path(), 
+            self.workingtree.base_revnum, self.mapping, self.id2path(file_id))
+        return annotater.get_annotated()
 
-    def iter_files_bytes(self, file_ids):
-        for file_id, identifier in file_ids:
-            cur_file = (self.get_file_text(file_id),)
-            yield identifier, cur_file
+    @property
+    def real_tree(self):
+        if self._real_tree is not None:
+            return self._real_tree
+        self._real_tree = self.workingtree.revision_tree(self.get_revision_id())
+        return self._real_tree
+
+    def all_file_ids(self):
+        return self.real_tree.all_file_ids()
+
+    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
+        # FIXME
+        try:
+            return self.inventory.iter_entries_by_dir(
+                specific_file_ids=specific_file_ids, yield_parents=yield_parents)
+        except BasisTreeIncomplete:
+            return self.real_tree.iter_entries_by_dir(
+                specific_file_ids=specific_file_ids,
+                yield_parents=yield_parents)
